from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="SessionResponse")


@_attrs_define
class SessionResponse:
    """Response from session creation endpoint.

    Attributes:
        session_token (str): Session token for authenticated requests
        expires_at (int): Unix timestamp when session expires
        role (str): User role (miner, validator, admin)
    """

    session_token: str
    expires_at: int
    role: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        session_token = self.session_token

        expires_at = self.expires_at

        role = self.role

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "session_token": session_token,
                "expires_at": expires_at,
                "role": role,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        session_token = d.pop("session_token")

        expires_at = d.pop("expires_at")

        role = d.pop("role")

        session_response = cls(
            session_token=session_token,
            expires_at=expires_at,
            role=role,
        )

        session_response.additional_properties = d
        return session_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="PendingEvaluationSummary")


@_attrs_define
class PendingEvaluationSummary:
    """Aggregate statistics for pending evaluations.

    Attributes:
        total_pending (int): Number of open work items
        total_runs_needed (int): Sum of remaining runs across all items
        total_runs_active (int): Sum of active runs across all items
    """

    total_pending: int
    total_runs_needed: int
    total_runs_active: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        total_pending = self.total_pending

        total_runs_needed = self.total_runs_needed

        total_runs_active = self.total_runs_active

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "total_pending": total_pending,
                "total_runs_needed": total_runs_needed,
                "total_runs_active": total_runs_active,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        total_pending = d.pop("total_pending")

        total_runs_needed = d.pop("total_runs_needed")

        total_runs_active = d.pop("total_runs_active")

        pending_evaluation_summary = cls(
            total_pending=total_pending,
            total_runs_needed=total_runs_needed,
            total_runs_active=total_runs_active,
        )

        pending_evaluation_summary.additional_properties = d
        return pending_evaluation_summary

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.top_agent_response_policy_type_0 import TopAgentResponsePolicyType0


T = TypeVar("T", bound="TopAgentResponse")


@_attrs_define
class TopAgentResponse:
    """Response for top agent endpoint.

    Attributes:
        suite_id (int): Active suite ID
        computed_at (datetime.datetime): When this was computed
        top_agent_version_id (None | Unset | UUID): Top agent version ID
        top_miner_hotkey (None | str | Unset): Top miner's hotkey
        top_score (float | None | Unset): Top score
        policy (None | TopAgentResponsePolicyType0 | Unset): Hysteresis/decay params
        challenge_threshold (float | None | Unset): Score required to dethrone
        emission_weight (float | None | Unset): Current emission weight (1.0=full)
        margin (float | None | Unset): Current challenge margin fraction
    """

    suite_id: int
    computed_at: datetime.datetime
    top_agent_version_id: None | Unset | UUID = UNSET
    top_miner_hotkey: None | str | Unset = UNSET
    top_score: float | None | Unset = UNSET
    policy: None | TopAgentResponsePolicyType0 | Unset = UNSET
    challenge_threshold: float | None | Unset = UNSET
    emission_weight: float | None | Unset = UNSET
    margin: float | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.top_agent_response_policy_type_0 import TopAgentResponsePolicyType0

        suite_id = self.suite_id

        computed_at = self.computed_at.isoformat()

        top_agent_version_id: None | str | Unset
        if isinstance(self.top_agent_version_id, Unset):
            top_agent_version_id = UNSET
        elif isinstance(self.top_agent_version_id, UUID):
            top_agent_version_id = str(self.top_agent_version_id)
        else:
            top_agent_version_id = self.top_agent_version_id

        top_miner_hotkey: None | str | Unset
        if isinstance(self.top_miner_hotkey, Unset):
            top_miner_hotkey = UNSET
        else:
            top_miner_hotkey = self.top_miner_hotkey

        top_score: float | None | Unset
        if isinstance(self.top_score, Unset):
            top_score = UNSET
        else:
            top_score = self.top_score

        policy: dict[str, Any] | None | Unset
        if isinstance(self.policy, Unset):
            policy = UNSET
        elif isinstance(self.policy, TopAgentResponsePolicyType0):
            policy = self.policy.to_dict()
        else:
            policy = self.policy

        challenge_threshold: float | None | Unset
        if isinstance(self.challenge_threshold, Unset):
            challenge_threshold = UNSET
        else:
            challenge_threshold = self.challenge_threshold

        emission_weight: float | None | Unset
        if isinstance(self.emission_weight, Unset):
            emission_weight = UNSET
        else:
            emission_weight = self.emission_weight

        margin: float | None | Unset
        if isinstance(self.margin, Unset):
            margin = UNSET
        else:
            margin = self.margin

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "suite_id": suite_id,
                "computed_at": computed_at,
            }
        )
        if top_agent_version_id is not UNSET:
            field_dict["top_agent_version_id"] = top_agent_version_id
        if top_miner_hotkey is not UNSET:
            field_dict["top_miner_hotkey"] = top_miner_hotkey
        if top_score is not UNSET:
            field_dict["top_score"] = top_score
        if policy is not UNSET:
            field_dict["policy"] = policy
        if challenge_threshold is not UNSET:
            field_dict["challenge_threshold"] = challenge_threshold
        if emission_weight is not UNSET:
            field_dict["emission_weight"] = emission_weight
        if margin is not UNSET:
            field_dict["margin"] = margin

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.top_agent_response_policy_type_0 import TopAgentResponsePolicyType0

        d = dict(src_dict)
        suite_id = d.pop("suite_id")

        computed_at = isoparse(d.pop("computed_at"))

        def _parse_top_agent_version_id(data: object) -> None | Unset | UUID:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                top_agent_version_id_type_0 = UUID(data)

                return top_agent_version_id_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UUID, data)

        top_agent_version_id = _parse_top_agent_version_id(d.pop("top_agent_version_id", UNSET))

        def _parse_top_miner_hotkey(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        top_miner_hotkey = _parse_top_miner_hotkey(d.pop("top_miner_hotkey", UNSET))

        def _parse_top_score(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        top_score = _parse_top_score(d.pop("top_score", UNSET))

        def _parse_policy(data: object) -> None | TopAgentResponsePolicyType0 | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                policy_type_0 = TopAgentResponsePolicyType0.from_dict(data)

                return policy_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | TopAgentResponsePolicyType0 | Unset, data)

        policy = _parse_policy(d.pop("policy", UNSET))

        def _parse_challenge_threshold(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        challenge_threshold = _parse_challenge_threshold(d.pop("challenge_threshold", UNSET))

        def _parse_emission_weight(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        emission_weight = _parse_emission_weight(d.pop("emission_weight", UNSET))

        def _parse_margin(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        margin = _parse_margin(d.pop("margin", UNSET))

        top_agent_response = cls(
            suite_id=suite_id,
            computed_at=computed_at,
            top_agent_version_id=top_agent_version_id,
            top_miner_hotkey=top_miner_hotkey,
            top_score=top_score,
            policy=policy,
            challenge_threshold=challenge_threshold,
            emission_weight=emission_weight,
            margin=margin,
        )

        top_agent_response.additional_properties = d
        return top_agent_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="ClaimWorkResponse")


@_attrs_define
class ClaimWorkResponse:
    """Response for successful work claim.

    Attributes:
        eval_run_id (UUID): Unique identifier for the evaluation run
        agent_version_id (UUID): Agent version to evaluate
        suite_id (int): Problem suite ID
        lease_expires_at (datetime.datetime): When the lease expires
        code_download_url (str): Presigned URL to download agent code
        chutes_access_token (None | str | Unset): Fresh Chutes access token for miner-funded inference
    """

    eval_run_id: UUID
    agent_version_id: UUID
    suite_id: int
    lease_expires_at: datetime.datetime
    code_download_url: str
    chutes_access_token: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        eval_run_id = str(self.eval_run_id)

        agent_version_id = str(self.agent_version_id)

        suite_id = self.suite_id

        lease_expires_at = self.lease_expires_at.isoformat()

        code_download_url = self.code_download_url

        chutes_access_token: None | str | Unset
        if isinstance(self.chutes_access_token, Unset):
            chutes_access_token = UNSET
        else:
            chutes_access_token = self.chutes_access_token

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "eval_run_id": eval_run_id,
                "agent_version_id": agent_version_id,
                "suite_id": suite_id,
                "lease_expires_at": lease_expires_at,
                "code_download_url": code_download_url,
            }
        )
        if chutes_access_token is not UNSET:
            field_dict["chutes_access_token"] = chutes_access_token

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        eval_run_id = UUID(d.pop("eval_run_id"))

        agent_version_id = UUID(d.pop("agent_version_id"))

        suite_id = d.pop("suite_id")

        lease_expires_at = isoparse(d.pop("lease_expires_at"))

        code_download_url = d.pop("code_download_url")

        def _parse_chutes_access_token(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        chutes_access_token = _parse_chutes_access_token(d.pop("chutes_access_token", UNSET))

        claim_work_response = cls(
            eval_run_id=eval_run_id,
            agent_version_id=agent_version_id,
            suite_id=suite_id,
            lease_expires_at=lease_expires_at,
            code_download_url=code_download_url,
            chutes_access_token=chutes_access_token,
        )

        claim_work_response.additional_properties = d
        return claim_work_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Auto-generated error type and code lists.

DO NOT EDIT — regenerate with: python3 scripts/extract-errors.py
"""

from __future__ import annotations

from typing import Literal

from .models import (
    AgentNotFoundError,
    AgentVersionNotFoundError,
    AlreadyInvalidatedError,
    ArtifactNotFoundError,
    ArtifactNotReleasedError,
    AtCapacityError,
    CodeAnalysisError,
    CooldownActiveError,
    EvalRunNotFoundError,
    FileTooLargeError,
    HTTPValidationError,
    InvalidAgentNameError,
    InvalidArtifactTypeError,
    InvalidFileError,
    InvalidProblemIdError,
    LeaseExpiredError,
    MinerNotFoundError,
    MissingParameterError,
    MissingScoreError,
    NoActiveSuiteError,
    NotRunOwnerError,
    ProblemNotFoundError,
    RateLimitExceededError,
    RunAlreadyCompleteError,
    ScoreBelowThresholdError,
    SuiteNotFoundError,
    ValidatorNotFoundError,
)

OroErrorCode = Literal[
    "AGENT_NOT_FOUND",
    "AGENT_VERSION_NOT_FOUND",
    "ALREADY_INVALIDATED",
    "ARTIFACT_NOT_FOUND",
    "ARTIFACT_NOT_RELEASED",
    "AT_CAPACITY",
    "CODE_ANALYSIS_ERROR",
    "COOLDOWN_ACTIVE",
    "EVAL_RUN_NOT_FOUND",
    "FILE_TOO_LARGE",
    "INVALID_AGENT_NAME",
    "INVALID_ARTIFACT_TYPE",
    "INVALID_FILE",
    "INVALID_PROBLEM_ID",
    "LEASE_EXPIRED",
    "MINER_NOT_FOUND",
    "MISSING_PARAMETER",
    "MISSING_SCORE",
    "NO_ACTIVE_SUITE",
    "NOT_RUN_OWNER",
    "PROBLEM_NOT_FOUND",
    "RATE_LIMIT_EXCEEDED",
    "RUN_ALREADY_COMPLETE",
    "SCORE_BELOW_THRESHOLD",
    "SUITE_NOT_FOUND",
    "VALIDATOR_NOT_FOUND",
]

ALL_ERROR_TYPES: tuple[type, ...] = (
    AgentNotFoundError,
    AgentVersionNotFoundError,
    AlreadyInvalidatedError,
    ArtifactNotFoundError,
    ArtifactNotReleasedError,
    AtCapacityError,
    CodeAnalysisError,
    CooldownActiveError,
    EvalRunNotFoundError,
    FileTooLargeError,
    HTTPValidationError,
    InvalidAgentNameError,
    InvalidArtifactTypeError,
    InvalidFileError,
    InvalidProblemIdError,
    LeaseExpiredError,
    MinerNotFoundError,
    MissingParameterError,
    MissingScoreError,
    NoActiveSuiteError,
    NotRunOwnerError,
    ProblemNotFoundError,
    RateLimitExceededError,
    RunAlreadyCompleteError,
    ScoreBelowThresholdError,
    SuiteNotFoundError,
    ValidatorNotFoundError,
)

__all__ = [
    "AgentNotFoundError",
    "AgentVersionNotFoundError",
    "AlreadyInvalidatedError",
    "ArtifactNotFoundError",
    "ArtifactNotReleasedError",
    "AtCapacityError",
    "CodeAnalysisError",
    "CooldownActiveError",
    "EvalRunNotFoundError",
    "FileTooLargeError",
    "HTTPValidationError",
    "InvalidAgentNameError",
    "InvalidArtifactTypeError",
    "InvalidFileError",
    "InvalidProblemIdError",
    "LeaseExpiredError",
    "MinerNotFoundError",
    "MissingParameterError",
    "MissingScoreError",
    "NoActiveSuiteError",
    "NotRunOwnerError",
    "ProblemNotFoundError",
    "RateLimitExceededError",
    "RunAlreadyCompleteError",
    "ScoreBelowThresholdError",
    "SuiteNotFoundError",
    "ValidatorNotFoundError",
    "OroErrorCode",
    "ALL_ERROR_TYPES",
]



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="SetTopRequest")


@_attrs_define
class SetTopRequest:
    """Request to set an agent version as the top agent for emissions.

    Attributes:
        suite_id (int | None | Unset): Suite ID (defaults to current suite)
        force (bool | Unset): Bypass the challenge threshold check (recorded in audit) Default: False.
    """

    suite_id: int | None | Unset = UNSET
    force: bool | Unset = False
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        suite_id: int | None | Unset
        if isinstance(self.suite_id, Unset):
            suite_id = UNSET
        else:
            suite_id = self.suite_id

        force = self.force

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if suite_id is not UNSET:
            field_dict["suite_id"] = suite_id
        if force is not UNSET:
            field_dict["force"] = force

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)

        def _parse_suite_id(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        suite_id = _parse_suite_id(d.pop("suite_id", UNSET))

        force = d.pop("force", UNSET)

        set_top_request = cls(
            suite_id=suite_id,
            force=force,
        )

        set_top_request.additional_properties = d
        return set_top_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.audit_events_response import AuditEventsResponse
from ...models.http_validation_error import HTTPValidationError
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    action: None | str | Unset = UNSET,
    target_type: None | str | Unset = UNSET,
    target_id: None | str | Unset = UNSET,
    actor_hotkey: None | str | Unset = UNSET,
    limit: int | Unset = 100,
    offset: int | Unset = 0,
) -> dict[str, Any]:

    params: dict[str, Any] = {}

    json_action: None | str | Unset
    if isinstance(action, Unset):
        json_action = UNSET
    else:
        json_action = action
    params["action"] = json_action

    json_target_type: None | str | Unset
    if isinstance(target_type, Unset):
        json_target_type = UNSET
    else:
        json_target_type = target_type
    params["target_type"] = json_target_type

    json_target_id: None | str | Unset
    if isinstance(target_id, Unset):
        json_target_id = UNSET
    else:
        json_target_id = target_id
    params["target_id"] = json_target_id

    json_actor_hotkey: None | str | Unset
    if isinstance(actor_hotkey, Unset):
        json_actor_hotkey = UNSET
    else:
        json_actor_hotkey = actor_hotkey
    params["actor_hotkey"] = json_actor_hotkey

    params["limit"] = limit

    params["offset"] = offset

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/v1/admin/audit-events",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> AuditEventsResponse | HTTPValidationError | None:
    if response.status_code == 200:
        response_200 = AuditEventsResponse.from_dict(response.json())

        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[AuditEventsResponse | HTTPValidationError]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    action: None | str | Unset = UNSET,
    target_type: None | str | Unset = UNSET,
    target_id: None | str | Unset = UNSET,
    actor_hotkey: None | str | Unset = UNSET,
    limit: int | Unset = 100,
    offset: int | Unset = 0,
) -> Response[AuditEventsResponse | HTTPValidationError]:
    """Get audit events

     Retrieve audit events with optional filtering and pagination.

    Args:
        action (None | str | Unset): Filter by action type (e.g., BAN_MINER)
        target_type (None | str | Unset): Filter by target type
        target_id (None | str | Unset): Filter by target ID
        actor_hotkey (None | str | Unset): Filter by actor hotkey
        limit (int | Unset): Number of events to return Default: 100.
        offset (int | Unset): Offset for pagination Default: 0.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[AuditEventsResponse | HTTPValidationError]
    """

    kwargs = _get_kwargs(
        action=action,
        target_type=target_type,
        target_id=target_id,
        actor_hotkey=actor_hotkey,
        limit=limit,
        offset=offset,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient | Client,
    action: None | str | Unset = UNSET,
    target_type: None | str | Unset = UNSET,
    target_id: None | str | Unset = UNSET,
    actor_hotkey: None | str | Unset = UNSET,
    limit: int | Unset = 100,
    offset: int | Unset = 0,
) -> AuditEventsResponse | HTTPValidationError | None:
    """Get audit events

     Retrieve audit events with optional filtering and pagination.

    Args:
        action (None | str | Unset): Filter by action type (e.g., BAN_MINER)
        target_type (None | str | Unset): Filter by target type
        target_id (None | str | Unset): Filter by target ID
        actor_hotkey (None | str | Unset): Filter by actor hotkey
        limit (int | Unset): Number of events to return Default: 100.
        offset (int | Unset): Offset for pagination Default: 0.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        AuditEventsResponse | HTTPValidationError
    """

    return sync_detailed(
        client=client,
        action=action,
        target_type=target_type,
        target_id=target_id,
        actor_hotkey=actor_hotkey,
        limit=limit,
        offset=offset,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    action: None | str | Unset = UNSET,
    target_type: None | str | Unset = UNSET,
    target_id: None | str | Unset = UNSET,
    actor_hotkey: None | str | Unset = UNSET,
    limit: int | Unset = 100,
    offset: int | Unset = 0,
) -> Response[AuditEventsResponse | HTTPValidationError]:
    """Get audit events

     Retrieve audit events with optional filtering and pagination.

    Args:
        action (None | str | Unset): Filter by action type (e.g., BAN_MINER)
        target_type (None | str | Unset): Filter by target type
        target_id (None | str | Unset): Filter by target ID
        actor_hotkey (None | str | Unset): Filter by actor hotkey
        limit (int | Unset): Number of events to return Default: 100.
        offset (int | Unset): Offset for pagination Default: 0.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[AuditEventsResponse | HTTPValidationError]
    """

    kwargs = _get_kwargs(
        action=action,
        target_type=target_type,
        target_id=target_id,
        actor_hotkey=actor_hotkey,
        limit=limit,
        offset=offset,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    action: None | str | Unset = UNSET,
    target_type: None | str | Unset = UNSET,
    target_id: None | str | Unset = UNSET,
    actor_hotkey: None | str | Unset = UNSET,
    limit: int | Unset = 100,
    offset: int | Unset = 0,
) -> AuditEventsResponse | HTTPValidationError | None:
    """Get audit events

     Retrieve audit events with optional filtering and pagination.

    Args:
        action (None | str | Unset): Filter by action type (e.g., BAN_MINER)
        target_type (None | str | Unset): Filter by target type
        target_id (None | str | Unset): Filter by target ID
        actor_hotkey (None | str | Unset): Filter by actor hotkey
        limit (int | Unset): Number of events to return Default: 100.
        offset (int | Unset): Offset for pagination Default: 0.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        AuditEventsResponse | HTTPValidationError
    """

    return (
        await asyncio_detailed(
            client=client,
            action=action,
            target_type=target_type,
            target_id=target_id,
            actor_hotkey=actor_hotkey,
            limit=limit,
            offset=offset,
        )
    ).parsed



from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.body_submit_agent import BodySubmitAgent
from ...models.code_analysis_error import CodeAnalysisError
from ...models.cooldown_active_error import CooldownActiveError
from ...models.file_too_large_error import FileTooLargeError
from ...models.invalid_agent_name_error import InvalidAgentNameError
from ...models.invalid_file_error import InvalidFileError
from ...models.no_active_suite_error import NoActiveSuiteError
from ...models.rate_limit_exceeded_error import RateLimitExceededError
from ...models.submit_agent_response import SubmitAgentResponse
from ...types import Response


def _get_kwargs(
    *,
    body: BodySubmitAgent,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/v1/miner/submit",
    }

    _kwargs["files"] = body.to_multipart()

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> (
    CodeAnalysisError
    | CooldownActiveError
    | RateLimitExceededError
    | FileTooLargeError
    | InvalidAgentNameError
    | InvalidFileError
    | NoActiveSuiteError
    | SubmitAgentResponse
    | None
):
    if response.status_code == 200:
        response_200 = SubmitAgentResponse.from_dict(response.json())

        return response_200

    if response.status_code == 400:

        def _parse_response_400(data: object) -> InvalidAgentNameError | InvalidFileError:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_400_type_0 = InvalidAgentNameError.from_dict(data)

                return response_400_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            response_400_type_1 = InvalidFileError.from_dict(data)

            return response_400_type_1

        response_400 = _parse_response_400(response.json())

        return response_400

    if response.status_code == 413:
        response_413 = FileTooLargeError.from_dict(response.json())

        return response_413

    if response.status_code == 422:
        response_422 = CodeAnalysisError.from_dict(response.json())

        return response_422

    if response.status_code == 429:

        def _parse_response_429(data: object) -> CooldownActiveError | RateLimitExceededError:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_429_type_0 = CooldownActiveError.from_dict(data)

                return response_429_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            response_429_type_1 = RateLimitExceededError.from_dict(data)

            return response_429_type_1

        response_429 = _parse_response_429(response.json())

        return response_429

    if response.status_code == 503:
        response_503 = NoActiveSuiteError.from_dict(response.json())

        return response_503

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[
    CodeAnalysisError
    | CooldownActiveError
    | RateLimitExceededError
    | FileTooLargeError
    | InvalidAgentNameError
    | InvalidFileError
    | NoActiveSuiteError
    | SubmitAgentResponse
]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: BodySubmitAgent,
) -> Response[
    CodeAnalysisError
    | CooldownActiveError
    | RateLimitExceededError
    | FileTooLargeError
    | InvalidAgentNameError
    | InvalidFileError
    | NoActiveSuiteError
    | SubmitAgentResponse
]:
    """Submit agent

     Submit an agent file for evaluation. Creates new agent if needed.

    Args:
        body (BodySubmitAgent):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[CodeAnalysisError | CooldownActiveError | RateLimitExceededError | FileTooLargeError | InvalidAgentNameError | InvalidFileError | NoActiveSuiteError | SubmitAgentResponse]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient | Client,
    body: BodySubmitAgent,
) -> (
    CodeAnalysisError
    | CooldownActiveError
    | RateLimitExceededError
    | FileTooLargeError
    | InvalidAgentNameError
    | InvalidFileError
    | NoActiveSuiteError
    | SubmitAgentResponse
    | None
):
    """Submit agent

     Submit an agent file for evaluation. Creates new agent if needed.

    Args:
        body (BodySubmitAgent):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        CodeAnalysisError | CooldownActiveError | RateLimitExceededError | FileTooLargeError | InvalidAgentNameError | InvalidFileError | NoActiveSuiteError | SubmitAgentResponse
    """

    return sync_detailed(
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: BodySubmitAgent,
) -> Response[
    CodeAnalysisError
    | CooldownActiveError
    | RateLimitExceededError
    | FileTooLargeError
    | InvalidAgentNameError
    | InvalidFileError
    | NoActiveSuiteError
    | SubmitAgentResponse
]:
    """Submit agent

     Submit an agent file for evaluation. Creates new agent if needed.

    Args:
        body (BodySubmitAgent):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[CodeAnalysisError | CooldownActiveError | RateLimitExceededError | FileTooLargeError | InvalidAgentNameError | InvalidFileError | NoActiveSuiteError | SubmitAgentResponse]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    body: BodySubmitAgent,
) -> (
    CodeAnalysisError
    | CooldownActiveError
    | RateLimitExceededError
    | FileTooLargeError
    | InvalidAgentNameError
    | InvalidFileError
    | NoActiveSuiteError
    | SubmitAgentResponse
    | None
):
    """Submit agent

     Submit an agent file for evaluation. Creates new agent if needed.

    Args:
        body (BodySubmitAgent):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        CodeAnalysisError | CooldownActiveError | RateLimitExceededError | FileTooLargeError | InvalidAgentNameError | InvalidFileError | NoActiveSuiteError | SubmitAgentResponse
    """

    return (
        await asyncio_detailed(
            client=client,
            body=body,
        )
    ).parsed



from enum import Enum


class EvaluationRunStatus(str, Enum):
    CANCELLED = "CANCELLED"
    CLAIMED = "CLAIMED"
    FAILED = "FAILED"
    RUNNING = "RUNNING"
    STALE = "STALE"
    SUCCESS = "SUCCESS"
    TIMED_OUT = "TIMED_OUT"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.leaderboard_entry import LeaderboardEntry
    from ..models.suite_public import SuitePublic


T = TypeVar("T", bound="LeaderboardResponse")


@_attrs_define
class LeaderboardResponse:
    """Response for leaderboard endpoint.

    Attributes:
        suite (SuitePublic): Public representation of a problem suite.
        entries (list[LeaderboardEntry]): Leaderboard entries
        total (int): Total entries
        limit (int): Page size
        offset (int): Page offset
        challenge_threshold (float | None | Unset): Score required to dethrone current top
    """

    suite: SuitePublic
    entries: list[LeaderboardEntry]
    total: int
    limit: int
    offset: int
    challenge_threshold: float | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        suite = self.suite.to_dict()

        entries = []
        for entries_item_data in self.entries:
            entries_item = entries_item_data.to_dict()
            entries.append(entries_item)

        total = self.total

        limit = self.limit

        offset = self.offset

        challenge_threshold: float | None | Unset
        if isinstance(self.challenge_threshold, Unset):
            challenge_threshold = UNSET
        else:
            challenge_threshold = self.challenge_threshold

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "suite": suite,
                "entries": entries,
                "total": total,
                "limit": limit,
                "offset": offset,
            }
        )
        if challenge_threshold is not UNSET:
            field_dict["challenge_threshold"] = challenge_threshold

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.leaderboard_entry import LeaderboardEntry
        from ..models.suite_public import SuitePublic

        d = dict(src_dict)
        suite = SuitePublic.from_dict(d.pop("suite"))

        entries = []
        _entries = d.pop("entries")
        for entries_item_data in _entries:
            entries_item = LeaderboardEntry.from_dict(entries_item_data)

            entries.append(entries_item)

        total = d.pop("total")

        limit = d.pop("limit")

        offset = d.pop("offset")

        def _parse_challenge_threshold(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        challenge_threshold = _parse_challenge_threshold(d.pop("challenge_threshold", UNSET))

        leaderboard_response = cls(
            suite=suite,
            entries=entries,
            total=total,
            limit=limit,
            offset=offset,
            challenge_threshold=challenge_threshold,
        )

        leaderboard_response.additional_properties = d
        return leaderboard_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="CancelResponse")


@_attrs_define
class CancelResponse:
    """Response for cancel operation.

    Attributes:
        agent_version_id (UUID): Agent version ID
        cancelled_runs (int): Number of runs cancelled
    """

    agent_version_id: UUID
    cancelled_runs: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        agent_version_id = str(self.agent_version_id)

        cancelled_runs = self.cancelled_runs

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "agent_version_id": agent_version_id,
                "cancelled_runs": cancelled_runs,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        agent_version_id = UUID(d.pop("agent_version_id"))

        cancelled_runs = d.pop("cancelled_runs")

        cancel_response = cls(
            agent_version_id=agent_version_id,
            cancelled_runs=cancelled_runs,
        )

        cancel_response.additional_properties = d
        return cancel_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Chutes OAuth 2.0 + PKCE authentication flow for miner-funded inference.

Handles the browser-based OAuth flow so miners can authenticate with Chutes
and pass their refresh token to the Backend during agent submission.

Usage:
    from oro_sdk.chutes_auth import start_auth_flow, load_cached_tokens

    # Check for cached token first
    tokens = load_cached_tokens()
    if not tokens:
        tokens = start_auth_flow()

    # tokens["refresh_token"] is passed to the Backend on submit
"""

from __future__ import annotations

import base64
import hashlib
import json
import os
import secrets
import threading
import webbrowser
from http.server import BaseHTTPRequestHandler, HTTPServer
from pathlib import Path
from typing import Any
from urllib.parse import parse_qs, urlencode, urlparse

import httpx


class ChutesAuthError(Exception):
    """Raised when Chutes OAuth authentication fails."""


# Default paths and URLs
DEFAULT_TOKEN_CACHE = Path.home() / ".oro" / "chutes_tokens.json"
DEFAULT_REDIRECT_URI = "http://localhost:8765/callback"
DEFAULT_AUTH_URL = "https://api.chutes.ai/idp/authorize"
DEFAULT_TOKEN_URL = "https://api.chutes.ai/idp/token"

# Public OAuth client ID for the ORO SDK — same for all miners.
# PKCE replaces client_secret for public clients (RFC 7636).
DEFAULT_CLIENT_ID = os.environ.get("CHUTES_CLIENT_ID", "cid_qstkn2eb4uszxski1rdqk062")


def generate_pkce_pair() -> tuple[str, str]:
    """Generate a PKCE code_verifier and code_challenge pair.

    Returns:
        Tuple of (code_verifier, code_challenge) strings.
    """
    code_verifier = secrets.token_urlsafe(64)
    digest = hashlib.sha256(code_verifier.encode()).digest()
    code_challenge = base64.urlsafe_b64encode(digest).rstrip(b"=").decode()
    return code_verifier, code_challenge


def _build_authorize_url(
    client_id: str,
    redirect_uri: str,
    code_challenge: str,
    state: str,
    authorize_url: str = DEFAULT_AUTH_URL,
    scope: str = "chutes:invoke",
) -> str:
    """Build the Chutes authorization URL with PKCE parameters."""
    params = {
        "response_type": "code",
        "client_id": client_id,
        "redirect_uri": redirect_uri,
        "code_challenge": code_challenge,
        "code_challenge_method": "S256",
        "state": state,
        "scope": scope,
    }
    return f"{authorize_url}?{urlencode(params)}"


def exchange_code(
    code: str,
    verifier: str,
    client_id: str,
    redirect_uri: str,
    token_url: str = DEFAULT_TOKEN_URL,
) -> dict[str, Any]:
    """Exchange an authorization code for access and refresh tokens.

    Uses PKCE (code_verifier) instead of client_secret per RFC 7636.
    """
    with httpx.Client(timeout=15.0) as client:
        response = client.post(
            token_url,
            data={
                "grant_type": "authorization_code",
                "code": code,
                "code_verifier": verifier,
                "client_id": client_id,
                "redirect_uri": redirect_uri,
            },
        )
        response.raise_for_status()
        return response.json()



def load_cached_tokens(path: Path = DEFAULT_TOKEN_CACHE) -> dict[str, Any] | None:
    """Load cached Chutes tokens from disk.

    Returns:
        Token dict if cache exists and is valid, None otherwise.
    """
    if not path.exists():
        return None
    try:
        data = json.loads(path.read_text())
        if "refresh_token" in data:
            return data
    except (json.JSONDecodeError, OSError):
        pass
    return None


def save_cached_tokens(tokens: dict[str, Any], path: Path = DEFAULT_TOKEN_CACHE) -> None:
    """Save tokens to the cache file with restricted permissions.

    Creates ~/.oro/ (0o700) if needed. File is created atomically with 0o600.
    """
    path.parent.mkdir(parents=True, exist_ok=True)
    os.chmod(str(path.parent), 0o700)
    fd = os.open(str(path), os.O_WRONLY | os.O_CREAT | os.O_TRUNC, 0o600)
    try:
        os.write(fd, json.dumps(tokens, indent=2).encode())
    finally:
        os.close(fd)


def start_auth_flow(
    client_id: str = DEFAULT_CLIENT_ID,
    redirect_uri: str = DEFAULT_REDIRECT_URI,
    authorize_url: str = DEFAULT_AUTH_URL,
    token_url: str = DEFAULT_TOKEN_URL,
) -> dict[str, Any]:
    """Run the full OAuth 2.0 + PKCE flow interactively.

    1. Generates PKCE pair and CSRF state token
    2. Opens browser to Chutes authorization page
    3. Starts a local HTTP server to receive the callback
    4. Exchanges the authorization code for tokens
    5. Caches the tokens to ~/.oro/chutes_tokens.json

    Args:
        client_id: OAuth client ID (default: ORO SDK public client).
        redirect_uri: Localhost callback URI.
        authorize_url: Chutes authorization endpoint.
        token_url: Chutes token endpoint.

    Returns:
        Dict with "access_token", "refresh_token", etc.
    """
    verifier, challenge = generate_pkce_pair()
    state = secrets.token_urlsafe(32)
    auth_url = _build_authorize_url(client_id, redirect_uri, challenge, state, authorize_url)

    parsed = urlparse(redirect_uri)
    port = parsed.port or 8765

    auth_code: dict[str, str | None] = {"code": None, "error": None}

    class CallbackHandler(BaseHTTPRequestHandler):
        def do_GET(self) -> None:
            query = parse_qs(urlparse(self.path).query)

            # Verify state to prevent CSRF
            received_state = query.get("state", [None])[0]
            if received_state != state:
                auth_code["error"] = "State mismatch — possible CSRF attack"
                self.send_response(400)
                self.send_header("Content-Type", "text/html")
                self.end_headers()
                self.wfile.write(b"<html><body><h2>State mismatch</h2></body></html>")
                threading.Thread(target=server.shutdown).start()
                return

            if "code" in query:
                auth_code["code"] = query["code"][0]
                self.send_response(200)
                self.send_header("Content-Type", "text/html")
                self.end_headers()
                self.wfile.write(
                    b"<html><body><h2>Authorization successful!</h2>"
                    b"<p>You can close this tab and return to the terminal.</p>"
                    b"</body></html>"
                )
            elif "error" in query:
                auth_code["error"] = query.get("error_description", query["error"])[0]
                self.send_response(400)
                self.send_header("Content-Type", "text/html")
                self.end_headers()
                self.wfile.write(
                    f"<html><body><h2>Authorization failed</h2>"
                    f"<p>{auth_code['error']}</p></body></html>".encode()
                )
            else:
                # Ignore favicon, preflight, etc.
                self.send_response(204)
                self.end_headers()
                return

            threading.Thread(target=server.shutdown).start()

        def log_message(self, format: str, *args: Any) -> None:
            pass  # Suppress HTTP server logs

    server = HTTPServer(("localhost", port), CallbackHandler)

    print("Opening browser for Chutes authentication...")
    print(f"If the browser doesn't open, visit: {auth_url}")
    webbrowser.open(auth_url)

    server_thread = threading.Thread(target=server.serve_forever)
    server_thread.start()
    server_thread.join(timeout=300)
    server.shutdown()

    if auth_code["error"]:
        raise ChutesAuthError(f"Authentication failed: {auth_code['error']}")

    if not auth_code["code"]:
        raise ChutesAuthError("Authentication timed out. Please try again.")

    print("Exchanging authorization code for tokens...")
    tokens = exchange_code(
        code=auth_code["code"],
        verifier=verifier,
        client_id=client_id,
        redirect_uri=redirect_uri,
        token_url=token_url,
    )

    save_cached_tokens(tokens)
    print("Chutes authentication successful! Token cached to ~/.oro/chutes_tokens.json")
    return tokens



from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.agent_version_not_found_error import AgentVersionNotFoundError
from ...models.artifact_download_request import ArtifactDownloadRequest
from ...models.artifact_download_response import ArtifactDownloadResponse
from ...models.artifact_not_found_error import ArtifactNotFoundError
from ...models.artifact_not_released_error import ArtifactNotReleasedError
from ...models.eval_run_not_found_error import EvalRunNotFoundError
from ...models.http_validation_error import HTTPValidationError
from ...models.invalid_artifact_type_error import InvalidArtifactTypeError
from ...models.missing_parameter_error import MissingParameterError
from ...models.suite_not_found_error import SuiteNotFoundError
from ...types import Response


def _get_kwargs(
    *,
    body: ArtifactDownloadRequest,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/v1/public/artifacts/download-url",
    }

    _kwargs["json"] = body.to_dict()

    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> (
    AgentVersionNotFoundError
    | ArtifactNotFoundError
    | ArtifactNotReleasedError
    | EvalRunNotFoundError
    | SuiteNotFoundError
    | ArtifactDownloadResponse
    | HTTPValidationError
    | InvalidArtifactTypeError
    | MissingParameterError
    | None
):
    if response.status_code == 200:
        response_200 = ArtifactDownloadResponse.from_dict(response.json())

        return response_200

    if response.status_code == 400:

        def _parse_response_400(data: object) -> InvalidArtifactTypeError | MissingParameterError:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_400_type_0 = MissingParameterError.from_dict(data)

                return response_400_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            response_400_type_1 = InvalidArtifactTypeError.from_dict(data)

            return response_400_type_1

        response_400 = _parse_response_400(response.json())

        return response_400

    if response.status_code == 404:

        def _parse_response_404(
            data: object,
        ) -> (
            AgentVersionNotFoundError
            | ArtifactNotFoundError
            | ArtifactNotReleasedError
            | EvalRunNotFoundError
            | SuiteNotFoundError
        ):
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_404_type_0 = AgentVersionNotFoundError.from_dict(data)

                return response_404_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_404_type_1 = SuiteNotFoundError.from_dict(data)

                return response_404_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_404_type_2 = ArtifactNotReleasedError.from_dict(data)

                return response_404_type_2
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_404_type_3 = ArtifactNotFoundError.from_dict(data)

                return response_404_type_3
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            response_404_type_4 = EvalRunNotFoundError.from_dict(data)

            return response_404_type_4

        response_404 = _parse_response_404(response.json())

        return response_404

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[
    AgentVersionNotFoundError
    | ArtifactNotFoundError
    | ArtifactNotReleasedError
    | EvalRunNotFoundError
    | SuiteNotFoundError
    | ArtifactDownloadResponse
    | HTTPValidationError
    | InvalidArtifactTypeError
    | MissingParameterError
]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: ArtifactDownloadRequest,
) -> Response[
    AgentVersionNotFoundError
    | ArtifactNotFoundError
    | ArtifactNotReleasedError
    | EvalRunNotFoundError
    | SuiteNotFoundError
    | ArtifactDownloadResponse
    | HTTPValidationError
    | InvalidArtifactTypeError
    | MissingParameterError
]:
    """Get artifact download URL

     Get a presigned URL to download released artifacts.

    Args:
        body (ArtifactDownloadRequest): Request for artifact download URL.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[AgentVersionNotFoundError | ArtifactNotFoundError | ArtifactNotReleasedError | EvalRunNotFoundError | SuiteNotFoundError | ArtifactDownloadResponse | HTTPValidationError | InvalidArtifactTypeError | MissingParameterError]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient | Client,
    body: ArtifactDownloadRequest,
) -> (
    AgentVersionNotFoundError
    | ArtifactNotFoundError
    | ArtifactNotReleasedError
    | EvalRunNotFoundError
    | SuiteNotFoundError
    | ArtifactDownloadResponse
    | HTTPValidationError
    | InvalidArtifactTypeError
    | MissingParameterError
    | None
):
    """Get artifact download URL

     Get a presigned URL to download released artifacts.

    Args:
        body (ArtifactDownloadRequest): Request for artifact download URL.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        AgentVersionNotFoundError | ArtifactNotFoundError | ArtifactNotReleasedError | EvalRunNotFoundError | SuiteNotFoundError | ArtifactDownloadResponse | HTTPValidationError | InvalidArtifactTypeError | MissingParameterError
    """

    return sync_detailed(
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: ArtifactDownloadRequest,
) -> Response[
    AgentVersionNotFoundError
    | ArtifactNotFoundError
    | ArtifactNotReleasedError
    | EvalRunNotFoundError
    | SuiteNotFoundError
    | ArtifactDownloadResponse
    | HTTPValidationError
    | InvalidArtifactTypeError
    | MissingParameterError
]:
    """Get artifact download URL

     Get a presigned URL to download released artifacts.

    Args:
        body (ArtifactDownloadRequest): Request for artifact download URL.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[AgentVersionNotFoundError | ArtifactNotFoundError | ArtifactNotReleasedError | EvalRunNotFoundError | SuiteNotFoundError | ArtifactDownloadResponse | HTTPValidationError | InvalidArtifactTypeError | MissingParameterError]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    body: ArtifactDownloadRequest,
) -> (
    AgentVersionNotFoundError
    | ArtifactNotFoundError
    | ArtifactNotReleasedError
    | EvalRunNotFoundError
    | SuiteNotFoundError
    | ArtifactDownloadResponse
    | HTTPValidationError
    | InvalidArtifactTypeError
    | MissingParameterError
    | None
):
    """Get artifact download URL

     Get a presigned URL to download released artifacts.

    Args:
        body (ArtifactDownloadRequest): Request for artifact download URL.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        AgentVersionNotFoundError | ArtifactNotFoundError | ArtifactNotReleasedError | EvalRunNotFoundError | SuiteNotFoundError | ArtifactDownloadResponse | HTTPValidationError | InvalidArtifactTypeError | MissingParameterError
    """

    return (
        await asyncio_detailed(
            client=client,
            body=body,
        )
    ).parsed



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="LeaderboardEntry")


@_attrs_define
class LeaderboardEntry:
    """Entry in the leaderboard.

    Attributes:
        rank (int): Leaderboard rank
        agent_version_id (UUID): Agent version ID
        agent_name (str): Agent name
        miner_hotkey (str): Miner hotkey
        version_number (int): Version number within this agent (v1, v2, etc.)
        final_score (float): Final score
        eligible_at (datetime.datetime | None | Unset): When eligibility was reached
        is_discarded (bool | Unset): Whether discarded by admin Default: False.
        is_miner_banned (bool | Unset): Whether miner is banned Default: False.
        is_current_top (bool | Unset): Current top agent for emissions Default: False.
        was_top (bool | Unset): Previously held top agent status Default: False.
        top_at (datetime.datetime | None | Unset): When this agent became top
    """

    rank: int
    agent_version_id: UUID
    agent_name: str
    miner_hotkey: str
    version_number: int
    final_score: float
    eligible_at: datetime.datetime | None | Unset = UNSET
    is_discarded: bool | Unset = False
    is_miner_banned: bool | Unset = False
    is_current_top: bool | Unset = False
    was_top: bool | Unset = False
    top_at: datetime.datetime | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        rank = self.rank

        agent_version_id = str(self.agent_version_id)

        agent_name = self.agent_name

        miner_hotkey = self.miner_hotkey

        version_number = self.version_number

        final_score = self.final_score

        eligible_at: None | str | Unset
        if isinstance(self.eligible_at, Unset):
            eligible_at = UNSET
        elif isinstance(self.eligible_at, datetime.datetime):
            eligible_at = self.eligible_at.isoformat()
        else:
            eligible_at = self.eligible_at

        is_discarded = self.is_discarded

        is_miner_banned = self.is_miner_banned

        is_current_top = self.is_current_top

        was_top = self.was_top

        top_at: None | str | Unset
        if isinstance(self.top_at, Unset):
            top_at = UNSET
        elif isinstance(self.top_at, datetime.datetime):
            top_at = self.top_at.isoformat()
        else:
            top_at = self.top_at

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "rank": rank,
                "agent_version_id": agent_version_id,
                "agent_name": agent_name,
                "miner_hotkey": miner_hotkey,
                "version_number": version_number,
                "final_score": final_score,
            }
        )
        if eligible_at is not UNSET:
            field_dict["eligible_at"] = eligible_at
        if is_discarded is not UNSET:
            field_dict["is_discarded"] = is_discarded
        if is_miner_banned is not UNSET:
            field_dict["is_miner_banned"] = is_miner_banned
        if is_current_top is not UNSET:
            field_dict["is_current_top"] = is_current_top
        if was_top is not UNSET:
            field_dict["was_top"] = was_top
        if top_at is not UNSET:
            field_dict["top_at"] = top_at

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        rank = d.pop("rank")

        agent_version_id = UUID(d.pop("agent_version_id"))

        agent_name = d.pop("agent_name")

        miner_hotkey = d.pop("miner_hotkey")

        version_number = d.pop("version_number")

        final_score = d.pop("final_score")

        def _parse_eligible_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                eligible_at_type_0 = isoparse(data)

                return eligible_at_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        eligible_at = _parse_eligible_at(d.pop("eligible_at", UNSET))

        is_discarded = d.pop("is_discarded", UNSET)

        is_miner_banned = d.pop("is_miner_banned", UNSET)

        is_current_top = d.pop("is_current_top", UNSET)

        was_top = d.pop("was_top", UNSET)

        def _parse_top_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                top_at_type_0 = isoparse(data)

                return top_at_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        top_at = _parse_top_at(d.pop("top_at", UNSET))

        leaderboard_entry = cls(
            rank=rank,
            agent_version_id=agent_version_id,
            agent_name=agent_name,
            miner_hotkey=miner_hotkey,
            version_number=version_number,
            final_score=final_score,
            eligible_at=eligible_at,
            is_discarded=is_discarded,
            is_miner_banned=is_miner_banned,
            is_current_top=is_current_top,
            was_top=was_top,
            top_at=top_at,
        )

        leaderboard_entry.additional_properties = d
        return leaderboard_entry

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Error classification utilities for the ORO SDK.

Provides helpers to classify HTTP responses, check transient errors,
and extract structured error details from API responses.

This is a custom (non-generated) module that survives SDK regeneration
because generate-python.sh only deletes known generated directories
(api/, models/) and files (client.py, errors.py, types.py, py.typed).
"""

from __future__ import annotations

from enum import Enum
from typing import Any, Union

from ._generated_errors import ALL_ERROR_TYPES, OroErrorCode
from .types import UNSET, Response, Unset


class ErrorCategory(str, Enum):
    """High-level error category derived from HTTP status codes."""

    NETWORK = "NETWORK"
    AUTH = "AUTH"
    NOT_FOUND = "NOT_FOUND"
    VALIDATION = "VALIDATION"
    CONFLICT = "CONFLICT"
    RATE_LIMITED = "RATE_LIMITED"
    SERVER = "SERVER"
    UNKNOWN = "UNKNOWN"


def classify_status(status_code: int) -> ErrorCategory:
    """Classify an HTTP status code into an ErrorCategory."""
    if status_code == 0:
        return ErrorCategory.NETWORK
    if status_code in (401, 403):
        return ErrorCategory.AUTH
    if status_code == 404:
        return ErrorCategory.NOT_FOUND
    if status_code == 409:
        return ErrorCategory.CONFLICT
    if status_code == 422:
        return ErrorCategory.VALIDATION
    if status_code == 429:
        return ErrorCategory.RATE_LIMITED
    if 500 <= status_code < 600:
        return ErrorCategory.SERVER
    if 400 <= status_code < 500:
        return ErrorCategory.VALIDATION
    return ErrorCategory.UNKNOWN


def classify_error(response: Response[Any]) -> ErrorCategory:
    """Classify a Response object into an ErrorCategory."""
    return classify_status(response.status_code.value)


def is_transient_status(status_code: int) -> bool:
    """Return True if the status code represents a transient (retryable) error.

    Transient statuses: 429 (rate limited), 5xx (server errors), 0 (network failures).
    """
    if status_code == 0:
        return True
    if status_code == 429:
        return True
    if 500 <= status_code < 600:
        return True
    return False


def is_transient(response: Response[Any]) -> bool:
    """Return True if the response represents a transient (retryable) error."""
    return is_transient_status(response.status_code.value)


def get_error_detail(parsed: Any) -> Union[str, None]:
    """Extract the detail string from a parsed error model.

    Returns None if detail is missing, not a string (e.g. Pydantic validation arrays),
    or the parsed value is None.
    """
    if parsed is None:
        return None
    detail = getattr(parsed, "detail", None)
    if isinstance(detail, Unset):
        return None
    if isinstance(detail, str):
        return detail
    return None


def get_error_code(parsed: Any) -> Union[str, None]:
    """Extract the error_code from a parsed error model.

    Returns None if the field is missing, is Unset, or the parsed value is None.
    """
    if parsed is None:
        return None
    code = getattr(parsed, "error_code", None)
    if isinstance(code, Unset):
        return None
    if isinstance(code, str):
        return code
    return None


def is_error(parsed: Any) -> bool:
    """Return True if the parsed object is one of the known SDK error types."""
    return isinstance(parsed, ALL_ERROR_TYPES)


__all__ = [
    "OroErrorCode",
    "ALL_ERROR_TYPES",
    "ErrorCategory",
    "classify_status",
    "classify_error",
    "is_transient_status",
    "is_transient",
    "get_error_detail",
    "get_error_code",
    "is_error",
]



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="PresignUploadRequest")


@_attrs_define
class PresignUploadRequest:
    """Request for presigned upload URL.

    Attributes:
        content_length (int): Size of the upload in bytes
        eval_run_id (UUID): Evaluation run ID
        problem_id (UUID): Problem ID
        content_type (str | Unset): Content type of the upload Default: 'application/json'.
    """

    content_length: int
    eval_run_id: UUID
    problem_id: UUID
    content_type: str | Unset = "application/json"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        content_length = self.content_length

        eval_run_id = str(self.eval_run_id)

        problem_id = str(self.problem_id)

        content_type = self.content_type

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "content_length": content_length,
                "eval_run_id": eval_run_id,
                "problem_id": problem_id,
            }
        )
        if content_type is not UNSET:
            field_dict["content_type"] = content_type

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        content_length = d.pop("content_length")

        eval_run_id = UUID(d.pop("eval_run_id"))

        problem_id = UUID(d.pop("problem_id"))

        content_type = d.pop("content_type", UNSET)

        presign_upload_request = cls(
            content_length=content_length,
            eval_run_id=eval_run_id,
            problem_id=problem_id,
            content_type=content_type,
        )

        presign_upload_request.additional_properties = d
        return presign_upload_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Tests for Bittensor authentication utilities."""

import re
import time
from unittest.mock import MagicMock

import httpx
import pytest

from oro_sdk.bittensor_auth import (
    AsyncSigningTransport,
    BittensorAuthClient,
    SigningTransport,
    generate_auth_headers,
    is_public_endpoint,
)
from oro_sdk.retry import AsyncRetryTransport, RetryTransport


def create_mock_wallet(hotkey: str = "5GrwvaEF5zXb26Fz9rcQpDWS57CtERHpNehXCPcNoHGKutQY") -> MagicMock:
    """Create a mock Bittensor wallet for testing."""
    wallet = MagicMock()
    wallet.hotkey.ss58_address = hotkey
    wallet.hotkey.sign.return_value = bytes.fromhex("ab" * 64)  # 64-byte signature
    return wallet


class TestGenerateAuthHeaders:
    """Tests for generate_auth_headers function."""

    def test_returns_all_required_headers(self):
        """Should return X-Hotkey, X-Signature, X-Nonce, X-Timestamp."""
        wallet = create_mock_wallet()

        headers = generate_auth_headers(wallet)

        assert "X-Hotkey" in headers
        assert "X-Signature" in headers
        assert "X-Nonce" in headers
        assert "X-Timestamp" in headers

    def test_hotkey_matches_wallet(self):
        """X-Hotkey should match wallet's SS58 address."""
        hotkey = "5FHneW46xGXgs5mUiveU4sbTyGBzmstUspZC92UhjJM694ty"
        wallet = create_mock_wallet(hotkey)

        headers = generate_auth_headers(wallet)

        assert headers["X-Hotkey"] == hotkey

    def test_signature_is_hex_with_0x_prefix(self):
        """X-Signature should be hex-encoded with 0x prefix."""
        wallet = create_mock_wallet()

        headers = generate_auth_headers(wallet)

        assert headers["X-Signature"].startswith("0x")
        # Remove 0x and verify it's valid hex
        hex_part = headers["X-Signature"][2:]
        assert re.match(r"^[0-9a-fA-F]+$", hex_part)

    def test_nonce_is_uuid_format(self):
        """X-Nonce should be a valid UUID."""
        wallet = create_mock_wallet()

        headers = generate_auth_headers(wallet)

        uuid_pattern = r"^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$"
        assert re.match(uuid_pattern, headers["X-Nonce"])

    def test_timestamp_is_recent_unix_epoch(self):
        """X-Timestamp should be a recent Unix timestamp."""
        wallet = create_mock_wallet()
        before = int(time.time())

        headers = generate_auth_headers(wallet)

        after = int(time.time())
        timestamp = int(headers["X-Timestamp"])
        assert before <= timestamp <= after

    def test_custom_nonce_is_used(self):
        """Should use provided nonce instead of generating one."""
        wallet = create_mock_wallet()
        custom_nonce = "my-custom-nonce-123"

        headers = generate_auth_headers(wallet, nonce=custom_nonce)

        assert headers["X-Nonce"] == custom_nonce

    def test_signs_correct_message_format(self):
        """Should sign message in format 'hotkey:timestamp:nonce'."""
        wallet = create_mock_wallet()
        custom_nonce = "test-nonce"

        generate_auth_headers(wallet, nonce=custom_nonce)

        # Verify sign was called
        wallet.hotkey.sign.assert_called_once()
        signed_bytes = wallet.hotkey.sign.call_args[0][0]
        signed_message = signed_bytes.decode()

        # Message format: hotkey:timestamp:nonce
        parts = signed_message.split(":")
        assert len(parts) == 3
        assert parts[0] == wallet.hotkey.ss58_address
        assert parts[1].isdigit()  # timestamp
        assert parts[2] == custom_nonce


class TestIsPublicEndpoint:
    """Tests for is_public_endpoint function."""

    def test_returns_true_for_public_paths(self):
        """Should return True for /v1/public/* paths."""
        assert is_public_endpoint("/v1/public/leaderboard") is True
        assert is_public_endpoint("/v1/public/suites/current") is True
        assert is_public_endpoint("/v1/public/agents/123") is True

    def test_returns_true_for_health_path(self):
        """Should return True for /health path."""
        assert is_public_endpoint("/health") is True

    def test_returns_true_for_auth_paths(self):
        """Should return True for /v1/auth/* paths."""
        assert is_public_endpoint("/v1/auth/challenge") is True
        assert is_public_endpoint("/v1/auth/session") is True
        assert is_public_endpoint("/v1/auth/logout") is True

    def test_returns_false_for_miner_paths(self):
        """Should return False for /v1/miner/* paths."""
        assert is_public_endpoint("/v1/miner/agents") is False
        assert is_public_endpoint("/v1/miner/submit") is False

    def test_returns_false_for_validator_paths(self):
        """Should return False for /v1/validator/* paths."""
        assert is_public_endpoint("/v1/validator/claim") is False
        assert is_public_endpoint("/v1/validator/results") is False

    def test_returns_false_for_admin_paths(self):
        """Should return False for /v1/admin/* paths."""
        assert is_public_endpoint("/v1/admin/ban") is False

    def test_handles_full_urls(self):
        """Should handle full URLs correctly."""
        assert is_public_endpoint("https://api.oro.ai/v1/public/leaderboard") is True
        assert is_public_endpoint("https://api.oro.ai/v1/miner/agents") is False
        assert is_public_endpoint("https://api.oro.ai/health") is True

    def test_handles_query_parameters(self):
        """Should handle URLs with query parameters."""
        assert is_public_endpoint("/v1/public/leaderboard?limit=10") is True
        assert is_public_endpoint("/v1/miner/agents?page=1") is False

    def test_returns_false_for_none_or_empty(self):
        """Should return False for None or empty paths."""
        assert is_public_endpoint(None) is False
        assert is_public_endpoint("") is False


class TestSigningTransport:
    """Tests for SigningTransport."""

    def test_adds_auth_headers_to_authenticated_endpoints(self):
        """Should add auth headers to authenticated endpoint requests."""
        wallet = create_mock_wallet()
        mock_wrapped = MagicMock()
        mock_wrapped.handle_request.return_value = httpx.Response(200)

        transport = SigningTransport(wallet, wrapped=mock_wrapped)
        request = httpx.Request("GET", "https://api.example.com/v1/miner/agents")

        transport.handle_request(request)

        # Verify auth headers were added
        assert "X-Hotkey" in request.headers
        assert "X-Signature" in request.headers
        assert "X-Nonce" in request.headers
        assert "X-Timestamp" in request.headers

    def test_skips_auth_headers_for_public_endpoints(self):
        """Should NOT add auth headers to public endpoint requests."""
        wallet = create_mock_wallet()
        mock_wrapped = MagicMock()
        mock_wrapped.handle_request.return_value = httpx.Response(200)

        transport = SigningTransport(wallet, wrapped=mock_wrapped)
        request = httpx.Request("GET", "https://api.example.com/v1/public/leaderboard")

        transport.handle_request(request)

        # Verify auth headers were NOT added
        assert "X-Hotkey" not in request.headers
        assert "X-Signature" not in request.headers
        assert "X-Nonce" not in request.headers
        assert "X-Timestamp" not in request.headers

    def test_skips_auth_headers_for_health_endpoint(self):
        """Should NOT add auth headers to health endpoint requests."""
        wallet = create_mock_wallet()
        mock_wrapped = MagicMock()
        mock_wrapped.handle_request.return_value = httpx.Response(200)

        transport = SigningTransport(wallet, wrapped=mock_wrapped)
        request = httpx.Request("GET", "https://api.example.com/health")

        transport.handle_request(request)

        # Verify auth headers were NOT added
        assert "X-Hotkey" not in request.headers

    def test_passes_request_to_wrapped_transport(self):
        """Should call the wrapped transport's handle_request."""
        wallet = create_mock_wallet()
        mock_wrapped = MagicMock()
        expected_response = httpx.Response(200, json={"status": "ok"})
        mock_wrapped.handle_request.return_value = expected_response

        transport = SigningTransport(wallet, wrapped=mock_wrapped)
        request = httpx.Request("GET", "https://api.example.com/test")

        response = transport.handle_request(request)

        mock_wrapped.handle_request.assert_called_once_with(request)
        assert response == expected_response


class TestAsyncSigningTransport:
    """Tests for AsyncSigningTransport."""

    @pytest.mark.asyncio
    async def test_adds_auth_headers_to_authenticated_endpoints(self):
        """Should add auth headers to authenticated async requests."""
        from unittest.mock import AsyncMock

        wallet = create_mock_wallet()
        mock_wrapped = MagicMock()
        mock_wrapped.handle_async_request = AsyncMock(return_value=httpx.Response(200))

        transport = AsyncSigningTransport(wallet, wrapped=mock_wrapped)
        request = httpx.Request("GET", "https://api.example.com/v1/miner/agents")

        await transport.handle_async_request(request)

        # Verify auth headers were added
        assert "X-Hotkey" in request.headers
        assert "X-Signature" in request.headers
        assert "X-Nonce" in request.headers
        assert "X-Timestamp" in request.headers

    @pytest.mark.asyncio
    async def test_skips_auth_headers_for_public_endpoints(self):
        """Should NOT add auth headers to public async requests."""
        from unittest.mock import AsyncMock

        wallet = create_mock_wallet()
        mock_wrapped = MagicMock()
        mock_wrapped.handle_async_request = AsyncMock(return_value=httpx.Response(200))

        transport = AsyncSigningTransport(wallet, wrapped=mock_wrapped)
        request = httpx.Request("GET", "https://api.example.com/v1/public/leaderboard")

        await transport.handle_async_request(request)

        # Verify auth headers were NOT added
        assert "X-Hotkey" not in request.headers
        assert "X-Signature" not in request.headers


class TestBittensorAuthClient:
    """Tests for BittensorAuthClient."""

    def test_creates_client_with_signing_transport(self):
        """Should create httpx client with RetryTransport wrapping SigningTransport."""
        wallet = create_mock_wallet()

        client = BittensorAuthClient(base_url="https://api.example.com", wallet=wallet)
        httpx_client = client.get_httpx_client()

        assert isinstance(httpx_client, httpx.Client)
        assert str(httpx_client.base_url) == "https://api.example.com"
        assert isinstance(httpx_client._transport, RetryTransport)
        assert isinstance(httpx_client._transport._wrapped, SigningTransport)

    def test_creates_async_client_with_async_signing_transport(self):
        """Should create async httpx client with AsyncRetryTransport wrapping AsyncSigningTransport."""
        wallet = create_mock_wallet()

        client = BittensorAuthClient(base_url="https://api.example.com", wallet=wallet)
        httpx_client = client.get_async_httpx_client()

        assert isinstance(httpx_client, httpx.AsyncClient)
        assert str(httpx_client.base_url) == "https://api.example.com"
        assert isinstance(httpx_client._transport, AsyncRetryTransport)
        assert isinstance(httpx_client._transport._wrapped, AsyncSigningTransport)

    def test_context_manager_sync(self):
        """Should work as sync context manager."""
        wallet = create_mock_wallet()
        client = BittensorAuthClient(base_url="https://api.example.com", wallet=wallet)

        with client as ctx:
            assert ctx is client

    @pytest.mark.asyncio
    async def test_context_manager_async(self):
        """Should work as async context manager."""
        wallet = create_mock_wallet()
        client = BittensorAuthClient(base_url="https://api.example.com", wallet=wallet)

        async with client as ctx:
            assert ctx is client

    def test_reuses_client_instance(self):
        """Should return same client on multiple get_httpx_client calls."""
        wallet = create_mock_wallet()
        client = BittensorAuthClient(base_url="https://api.example.com", wallet=wallet)

        client1 = client.get_httpx_client()
        client2 = client.get_httpx_client()

        assert client1 is client2



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.problem_progress_entry import ProblemProgressEntry


T = TypeVar("T", bound="AgentVersionProblemsResponse")


@_attrs_define
class AgentVersionProblemsResponse:
    """Response for agent version problem progress.

    Attributes:
        agent_version_id (UUID): Agent version ID
        problems (list[ProblemProgressEntry]): Problem progress list
        suite_id (int | None | Unset): Suite ID
    """

    agent_version_id: UUID
    problems: list[ProblemProgressEntry]
    suite_id: int | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        agent_version_id = str(self.agent_version_id)

        problems = []
        for problems_item_data in self.problems:
            problems_item = problems_item_data.to_dict()
            problems.append(problems_item)

        suite_id: int | None | Unset
        if isinstance(self.suite_id, Unset):
            suite_id = UNSET
        else:
            suite_id = self.suite_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "agent_version_id": agent_version_id,
                "problems": problems,
            }
        )
        if suite_id is not UNSET:
            field_dict["suite_id"] = suite_id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.problem_progress_entry import ProblemProgressEntry

        d = dict(src_dict)
        agent_version_id = UUID(d.pop("agent_version_id"))

        problems = []
        _problems = d.pop("problems")
        for problems_item_data in _problems:
            problems_item = ProblemProgressEntry.from_dict(problems_item_data)

            problems.append(problems_item)

        def _parse_suite_id(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        suite_id = _parse_suite_id(d.pop("suite_id", UNSET))

        agent_version_problems_response = cls(
            agent_version_id=agent_version_id,
            problems=problems,
            suite_id=suite_id,
        )

        agent_version_problems_response.additional_properties = d
        return agent_version_problems_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.audit_event_entry_details_type_0 import AuditEventEntryDetailsType0


T = TypeVar("T", bound="AuditEventEntry")


@_attrs_define
class AuditEventEntry:
    """Audit event log entry.

    Attributes:
        event_id (UUID): Event ID
        action (str): Action performed
        target_type (str): Type of target entity
        target_id (str): Target entity ID
        actor_hotkey (str): Admin who performed the action
        created_at (datetime.datetime): When the event occurred
        reason (None | str | Unset): Reason provided
        details (AuditEventEntryDetailsType0 | None | Unset): Additional context
    """

    event_id: UUID
    action: str
    target_type: str
    target_id: str
    actor_hotkey: str
    created_at: datetime.datetime
    reason: None | str | Unset = UNSET
    details: AuditEventEntryDetailsType0 | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.audit_event_entry_details_type_0 import AuditEventEntryDetailsType0

        event_id = str(self.event_id)

        action = self.action

        target_type = self.target_type

        target_id = self.target_id

        actor_hotkey = self.actor_hotkey

        created_at = self.created_at.isoformat()

        reason: None | str | Unset
        if isinstance(self.reason, Unset):
            reason = UNSET
        else:
            reason = self.reason

        details: dict[str, Any] | None | Unset
        if isinstance(self.details, Unset):
            details = UNSET
        elif isinstance(self.details, AuditEventEntryDetailsType0):
            details = self.details.to_dict()
        else:
            details = self.details

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "event_id": event_id,
                "action": action,
                "target_type": target_type,
                "target_id": target_id,
                "actor_hotkey": actor_hotkey,
                "created_at": created_at,
            }
        )
        if reason is not UNSET:
            field_dict["reason"] = reason
        if details is not UNSET:
            field_dict["details"] = details

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.audit_event_entry_details_type_0 import AuditEventEntryDetailsType0

        d = dict(src_dict)
        event_id = UUID(d.pop("event_id"))

        action = d.pop("action")

        target_type = d.pop("target_type")

        target_id = d.pop("target_id")

        actor_hotkey = d.pop("actor_hotkey")

        created_at = isoparse(d.pop("created_at"))

        def _parse_reason(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        reason = _parse_reason(d.pop("reason", UNSET))

        def _parse_details(data: object) -> AuditEventEntryDetailsType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                details_type_0 = AuditEventEntryDetailsType0.from_dict(data)

                return details_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(AuditEventEntryDetailsType0 | None | Unset, data)

        details = _parse_details(d.pop("details", UNSET))

        audit_event_entry = cls(
            event_id=event_id,
            action=action,
            target_type=target_type,
            target_id=target_id,
            actor_hotkey=actor_hotkey,
            created_at=created_at,
            reason=reason,
            details=details,
        )

        audit_event_entry.additional_properties = d
        return audit_event_entry

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.agent_version_state import AgentVersionState
from ..types import UNSET, Unset

T = TypeVar("T", bound="AgentVersionHistoryEntry")


@_attrs_define
class AgentVersionHistoryEntry:
    """Entry in agent version history list.

    Attributes:
        agent_version_id (UUID): Version ID
        version_number (int): Version number within this agent (v1, v2, etc.)
        submitted_at (datetime.datetime): Submission timestamp
        state (AgentVersionState): State of an agent version evaluation.
        final_score (float | None | Unset): Final score if eligible
    """

    agent_version_id: UUID
    version_number: int
    submitted_at: datetime.datetime
    state: AgentVersionState
    final_score: float | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        agent_version_id = str(self.agent_version_id)

        version_number = self.version_number

        submitted_at = self.submitted_at.isoformat()

        state = self.state.value

        final_score: float | None | Unset
        if isinstance(self.final_score, Unset):
            final_score = UNSET
        else:
            final_score = self.final_score

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "agent_version_id": agent_version_id,
                "version_number": version_number,
                "submitted_at": submitted_at,
                "state": state,
            }
        )
        if final_score is not UNSET:
            field_dict["final_score"] = final_score

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        agent_version_id = UUID(d.pop("agent_version_id"))

        version_number = d.pop("version_number")

        submitted_at = isoparse(d.pop("submitted_at"))

        state = AgentVersionState(d.pop("state"))

        def _parse_final_score(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        final_score = _parse_final_score(d.pop("final_score", UNSET))

        agent_version_history_entry = cls(
            agent_version_id=agent_version_id,
            version_number=version_number,
            submitted_at=submitted_at,
            state=state,
            final_score=final_score,
        )

        agent_version_history_entry.additional_properties = d
        return agent_version_history_entry

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="DiscardRequest")


@_attrs_define
class DiscardRequest:
    """Request to discard an agent version.

    Attributes:
        reason (str): Reason for discarding
        suite_id (int | None | Unset): Suite ID (defaults to current suite)
    """

    reason: str
    suite_id: int | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        reason = self.reason

        suite_id: int | None | Unset
        if isinstance(self.suite_id, Unset):
            suite_id = UNSET
        else:
            suite_id = self.suite_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "reason": reason,
            }
        )
        if suite_id is not UNSET:
            field_dict["suite_id"] = suite_id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        reason = d.pop("reason")

        def _parse_suite_id(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        suite_id = _parse_suite_id(d.pop("suite_id", UNSET))

        discard_request = cls(
            reason=reason,
            suite_id=suite_id,
        )

        discard_request.additional_properties = d
        return discard_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.problem_status import ProblemStatus
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.problem_progress_update_score_components_summary_type_0 import (
        ProblemProgressUpdateScoreComponentsSummaryType0,
    )


T = TypeVar("T", bound="ProblemProgressUpdate")


@_attrs_define
class ProblemProgressUpdate:
    """Progress update for a single problem.

    Attributes:
        problem_id (UUID): Problem ID
        status (ProblemStatus): Status of problem evaluation.
        score (float | None | Unset): Score if available
        score_components_summary (None | ProblemProgressUpdateScoreComponentsSummaryType0 | Unset): Score breakdown if
            available
        logs_s3_key (None | str | Unset): S3 key for this problem's logs
    """

    problem_id: UUID
    status: ProblemStatus
    score: float | None | Unset = UNSET
    score_components_summary: None | ProblemProgressUpdateScoreComponentsSummaryType0 | Unset = UNSET
    logs_s3_key: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.problem_progress_update_score_components_summary_type_0 import (
            ProblemProgressUpdateScoreComponentsSummaryType0,
        )

        problem_id = str(self.problem_id)

        status = self.status.value

        score: float | None | Unset
        if isinstance(self.score, Unset):
            score = UNSET
        else:
            score = self.score

        score_components_summary: dict[str, Any] | None | Unset
        if isinstance(self.score_components_summary, Unset):
            score_components_summary = UNSET
        elif isinstance(self.score_components_summary, ProblemProgressUpdateScoreComponentsSummaryType0):
            score_components_summary = self.score_components_summary.to_dict()
        else:
            score_components_summary = self.score_components_summary

        logs_s3_key: None | str | Unset
        if isinstance(self.logs_s3_key, Unset):
            logs_s3_key = UNSET
        else:
            logs_s3_key = self.logs_s3_key

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "problem_id": problem_id,
                "status": status,
            }
        )
        if score is not UNSET:
            field_dict["score"] = score
        if score_components_summary is not UNSET:
            field_dict["score_components_summary"] = score_components_summary
        if logs_s3_key is not UNSET:
            field_dict["logs_s3_key"] = logs_s3_key

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.problem_progress_update_score_components_summary_type_0 import (
            ProblemProgressUpdateScoreComponentsSummaryType0,
        )

        d = dict(src_dict)
        problem_id = UUID(d.pop("problem_id"))

        status = ProblemStatus(d.pop("status"))

        def _parse_score(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        score = _parse_score(d.pop("score", UNSET))

        def _parse_score_components_summary(
            data: object,
        ) -> None | ProblemProgressUpdateScoreComponentsSummaryType0 | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                score_components_summary_type_0 = ProblemProgressUpdateScoreComponentsSummaryType0.from_dict(data)

                return score_components_summary_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | ProblemProgressUpdateScoreComponentsSummaryType0 | Unset, data)

        score_components_summary = _parse_score_components_summary(d.pop("score_components_summary", UNSET))

        def _parse_logs_s3_key(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        logs_s3_key = _parse_logs_s3_key(d.pop("logs_s3_key", UNSET))

        problem_progress_update = cls(
            problem_id=problem_id,
            status=status,
            score=score,
            score_components_summary=score_components_summary,
            logs_s3_key=logs_s3_key,
        )

        problem_progress_update.additional_properties = d
        return problem_progress_update

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.artifact_type import ArtifactType
from ..types import UNSET, Unset

T = TypeVar("T", bound="ArtifactDownloadRequest")


@_attrs_define
class ArtifactDownloadRequest:
    """Request for artifact download URL.

    Attributes:
        artifact_type (ArtifactType): Type of artifact for download.
        agent_version_id (UUID): Agent version ID
        eval_run_id (None | Unset | UUID): Run ID for run-scoped artifacts
        problem_id (None | Unset | UUID): Problem ID for per-problem artifact downloads
    """

    artifact_type: ArtifactType
    agent_version_id: UUID
    eval_run_id: None | Unset | UUID = UNSET
    problem_id: None | Unset | UUID = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        artifact_type = self.artifact_type.value

        agent_version_id = str(self.agent_version_id)

        eval_run_id: None | str | Unset
        if isinstance(self.eval_run_id, Unset):
            eval_run_id = UNSET
        elif isinstance(self.eval_run_id, UUID):
            eval_run_id = str(self.eval_run_id)
        else:
            eval_run_id = self.eval_run_id

        problem_id: None | str | Unset
        if isinstance(self.problem_id, Unset):
            problem_id = UNSET
        elif isinstance(self.problem_id, UUID):
            problem_id = str(self.problem_id)
        else:
            problem_id = self.problem_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "artifact_type": artifact_type,
                "agent_version_id": agent_version_id,
            }
        )
        if eval_run_id is not UNSET:
            field_dict["eval_run_id"] = eval_run_id
        if problem_id is not UNSET:
            field_dict["problem_id"] = problem_id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        artifact_type = ArtifactType(d.pop("artifact_type"))

        agent_version_id = UUID(d.pop("agent_version_id"))

        def _parse_eval_run_id(data: object) -> None | Unset | UUID:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                eval_run_id_type_0 = UUID(data)

                return eval_run_id_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UUID, data)

        eval_run_id = _parse_eval_run_id(d.pop("eval_run_id", UNSET))

        def _parse_problem_id(data: object) -> None | Unset | UUID:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                problem_id_type_0 = UUID(data)

                return problem_id_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UUID, data)

        problem_id = _parse_problem_id(d.pop("problem_id", UNSET))

        artifact_download_request = cls(
            artifact_type=artifact_type,
            agent_version_id=agent_version_id,
            eval_run_id=eval_run_id,
            problem_id=problem_id,
        )

        artifact_download_request.additional_properties = d
        return artifact_download_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.terminal_status import TerminalStatus
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.complete_run_request_score_components_type_0 import CompleteRunRequestScoreComponentsType0


T = TypeVar("T", bound="CompleteRunRequest")


@_attrs_define
class CompleteRunRequest:
    """Request to complete an evaluation run.

    Attributes:
        terminal_status (TerminalStatus): Terminal statuses for evaluation runs.
        validator_score (float | None | Unset): Score (required if SUCCESS)
        score_components (CompleteRunRequestScoreComponentsType0 | None | Unset): Score breakdown (required if SUCCESS)
        results_s3_key (None | str | Unset): S3 key for run-level logs bundle (optional, per-problem logs preferred)
        failure_reason (None | str | Unset): Reason for failure (e.g., 'No Chutes token', 'Sandbox execution failed')
    """

    terminal_status: TerminalStatus
    validator_score: float | None | Unset = UNSET
    score_components: CompleteRunRequestScoreComponentsType0 | None | Unset = UNSET
    results_s3_key: None | str | Unset = UNSET
    failure_reason: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.complete_run_request_score_components_type_0 import CompleteRunRequestScoreComponentsType0

        terminal_status = self.terminal_status.value

        validator_score: float | None | Unset
        if isinstance(self.validator_score, Unset):
            validator_score = UNSET
        else:
            validator_score = self.validator_score

        score_components: dict[str, Any] | None | Unset
        if isinstance(self.score_components, Unset):
            score_components = UNSET
        elif isinstance(self.score_components, CompleteRunRequestScoreComponentsType0):
            score_components = self.score_components.to_dict()
        else:
            score_components = self.score_components

        results_s3_key: None | str | Unset
        if isinstance(self.results_s3_key, Unset):
            results_s3_key = UNSET
        else:
            results_s3_key = self.results_s3_key

        failure_reason: None | str | Unset
        if isinstance(self.failure_reason, Unset):
            failure_reason = UNSET
        else:
            failure_reason = self.failure_reason

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "terminal_status": terminal_status,
            }
        )
        if validator_score is not UNSET:
            field_dict["validator_score"] = validator_score
        if score_components is not UNSET:
            field_dict["score_components"] = score_components
        if results_s3_key is not UNSET:
            field_dict["results_s3_key"] = results_s3_key
        if failure_reason is not UNSET:
            field_dict["failure_reason"] = failure_reason

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.complete_run_request_score_components_type_0 import CompleteRunRequestScoreComponentsType0

        d = dict(src_dict)
        terminal_status = TerminalStatus(d.pop("terminal_status"))

        def _parse_validator_score(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        validator_score = _parse_validator_score(d.pop("validator_score", UNSET))

        def _parse_score_components(data: object) -> CompleteRunRequestScoreComponentsType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                score_components_type_0 = CompleteRunRequestScoreComponentsType0.from_dict(data)

                return score_components_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(CompleteRunRequestScoreComponentsType0 | None | Unset, data)

        score_components = _parse_score_components(d.pop("score_components", UNSET))

        def _parse_results_s3_key(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        results_s3_key = _parse_results_s3_key(d.pop("results_s3_key", UNSET))

        def _parse_failure_reason(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        failure_reason = _parse_failure_reason(d.pop("failure_reason", UNSET))

        complete_run_request = cls(
            terminal_status=terminal_status,
            validator_score=validator_score,
            score_components=score_components,
            results_s3_key=results_s3_key,
            failure_reason=failure_reason,
        )

        complete_run_request.additional_properties = d
        return complete_run_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Tests for SDK client classes."""

import httpx
import pytest

from oro_sdk.client import AuthenticatedClient, Client


class TestClient:
    """Tests for the base Client class."""

    def test_creates_with_base_url(self):
        """Should create client with specified base URL."""
        client = Client(base_url="https://api.example.com")
        httpx_client = client.get_httpx_client()

        assert str(httpx_client.base_url) == "https://api.example.com"

    def test_with_headers_adds_headers(self):
        """Should add headers to client."""
        client = Client(base_url="https://api.example.com")
        client = client.with_headers({"X-Custom": "value"})

        httpx_client = client.get_httpx_client()
        assert httpx_client.headers["X-Custom"] == "value"

    def test_with_cookies_adds_cookies(self):
        """Should add cookies to client."""
        client = Client(base_url="https://api.example.com")
        client = client.with_cookies({"session": "abc123"})

        httpx_client = client.get_httpx_client()
        assert httpx_client.cookies["session"] == "abc123"

    def test_with_timeout_sets_timeout(self):
        """Should set timeout on client."""
        client = Client(base_url="https://api.example.com")
        timeout = httpx.Timeout(30.0)
        client = client.with_timeout(timeout)

        httpx_client = client.get_httpx_client()
        assert httpx_client.timeout == timeout

    def test_context_manager_sync(self):
        """Should work as sync context manager."""
        client = Client(base_url="https://api.example.com")

        with client as ctx:
            assert ctx is client

    @pytest.mark.asyncio
    async def test_context_manager_async(self):
        """Should work as async context manager."""
        client = Client(base_url="https://api.example.com")

        async with client as ctx:
            assert ctx is client

    def test_reuses_client_instance(self):
        """Should return same client on multiple get_httpx_client calls."""
        client = Client(base_url="https://api.example.com")

        client1 = client.get_httpx_client()
        client2 = client.get_httpx_client()

        assert client1 is client2

    def test_reuses_async_client_instance(self):
        """Should return same async client on multiple calls."""
        client = Client(base_url="https://api.example.com")

        client1 = client.get_async_httpx_client()
        client2 = client.get_async_httpx_client()

        assert client1 is client2

    def test_set_httpx_client_overrides(self):
        """Should allow manually setting the httpx client."""
        client = Client(base_url="https://api.example.com")
        custom_httpx = httpx.Client(base_url="https://custom.example.com")

        client.set_httpx_client(custom_httpx)

        assert client.get_httpx_client() is custom_httpx


class TestAuthenticatedClient:
    """Tests for the AuthenticatedClient class."""

    def test_creates_with_token(self):
        """Should create client with auth token in headers."""
        client = AuthenticatedClient(
            base_url="https://api.example.com",
            token="my-secret-token",
        )
        httpx_client = client.get_httpx_client()

        assert httpx_client.headers["Authorization"] == "Bearer my-secret-token"

    def test_custom_prefix(self):
        """Should use custom auth prefix."""
        client = AuthenticatedClient(
            base_url="https://api.example.com",
            token="my-token",
            prefix="Token",
        )
        httpx_client = client.get_httpx_client()

        assert httpx_client.headers["Authorization"] == "Token my-token"

    def test_no_prefix(self):
        """Should work without prefix."""
        client = AuthenticatedClient(
            base_url="https://api.example.com",
            token="my-token",
            prefix="",
        )
        httpx_client = client.get_httpx_client()

        assert httpx_client.headers["Authorization"] == "my-token"

    def test_custom_header_name(self):
        """Should use custom auth header name."""
        client = AuthenticatedClient(
            base_url="https://api.example.com",
            token="my-token",
            auth_header_name="X-API-Key",
            prefix="",
        )
        httpx_client = client.get_httpx_client()

        assert httpx_client.headers["X-API-Key"] == "my-token"
        assert "Authorization" not in httpx_client.headers

    def test_context_manager_sync(self):
        """Should work as sync context manager."""
        client = AuthenticatedClient(
            base_url="https://api.example.com",
            token="my-token",
        )

        with client as ctx:
            assert ctx is client

    @pytest.mark.asyncio
    async def test_context_manager_async(self):
        """Should work as async context manager."""
        client = AuthenticatedClient(
            base_url="https://api.example.com",
            token="my-token",
        )

        async with client as ctx:
            assert ctx is client



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.audit_event_entry import AuditEventEntry


T = TypeVar("T", bound="AuditEventsResponse")


@_attrs_define
class AuditEventsResponse:
    """Response for audit events list.

    Attributes:
        events (list[AuditEventEntry]): Audit events
        total (int): Total events matching filter
        limit (int): Page size
        offset (int): Page offset
    """

    events: list[AuditEventEntry]
    total: int
    limit: int
    offset: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        events = []
        for events_item_data in self.events:
            events_item = events_item_data.to_dict()
            events.append(events_item)

        total = self.total

        limit = self.limit

        offset = self.offset

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "events": events,
                "total": total,
                "limit": limit,
                "offset": offset,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.audit_event_entry import AuditEventEntry

        d = dict(src_dict)
        events = []
        _events = d.pop("events")
        for events_item_data in _events:
            events_item = AuditEventEntry.from_dict(events_item_data)

            events.append(events_item)

        total = d.pop("total")

        limit = d.pop("limit")

        offset = d.pop("offset")

        audit_events_response = cls(
            events=events,
            total=total,
            limit=limit,
            offset=offset,
        )

        audit_events_response.additional_properties = d
        return audit_events_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Tests for retry middleware."""

from unittest.mock import AsyncMock, MagicMock, patch

import httpx
import pytest

from oro_sdk.retry import (
    AsyncRetryTransport,
    RetryConfig,
    RetryContext,
    RetryTransport,
    compute_delay,
    parse_retry_after,
)

# ---------------------------------------------------------------------------
# parse_retry_after
# ---------------------------------------------------------------------------


class TestParseRetryAfter:
    def test_returns_none_when_header_missing(self):
        assert parse_retry_after(httpx.Response(429)) is None

    def test_parses_integer_seconds(self):
        assert parse_retry_after(httpx.Response(429, headers={"Retry-After": "120"})) == 120.0

    def test_parses_zero_seconds(self):
        assert parse_retry_after(httpx.Response(429, headers={"Retry-After": "0"})) == 0.0

    def test_parses_http_date(self):
        from datetime import datetime, timedelta, timezone
        from email.utils import format_datetime

        future = datetime.now(timezone.utc) + timedelta(seconds=60)
        resp = httpx.Response(429, headers={"Retry-After": format_datetime(future, usegmt=True)})
        result = parse_retry_after(resp)
        assert result is not None and 50.0 < result <= 65.0

    def test_returns_zero_for_past_date(self):
        from datetime import datetime, timedelta, timezone
        from email.utils import format_datetime

        past = datetime.now(timezone.utc) - timedelta(seconds=60)
        assert parse_retry_after(
            httpx.Response(429, headers={"Retry-After": format_datetime(past, usegmt=True)})
        ) == 0.0

    def test_returns_none_for_unparseable(self):
        assert parse_retry_after(httpx.Response(429, headers={"Retry-After": "not-valid"})) is None


# ---------------------------------------------------------------------------
# compute_delay
# ---------------------------------------------------------------------------


class TestComputeDelay:
    @pytest.mark.parametrize(
        ("attempt", "expected"),
        [(0, 1.0), (1, 2.0), (2, 4.0), (3, 8.0)],
    )
    def test_exponential_backoff(self, attempt: int, expected: float):
        assert compute_delay(attempt, 1.0, 30.0, False) == expected

    def test_caps_at_max_delay(self):
        assert compute_delay(10, 1.0, 30.0, False) == 30.0

    def test_jitter_produces_varied_delays(self):
        results = {compute_delay(2, 1.0, 30.0, True) for _ in range(100)}
        assert all(0 <= d < 4.0 for d in results)
        assert len(results) > 1

    def test_uses_retry_after_when_provided(self):
        assert compute_delay(0, 1.0, 30.0, False, retry_after_s=5.0) == 5.0

    def test_caps_retry_after_at_max(self):
        assert compute_delay(0, 1.0, 30.0, False, retry_after_s=60.0) == 30.0


# ---------------------------------------------------------------------------
# Helpers shared across sync/async transport tests
# ---------------------------------------------------------------------------

RETRYABLE_STATUSES = [429, 502, 503, 504]
NON_RETRYABLE_STATUSES = [400, 401, 404, 422]


def _make_config(**overrides) -> RetryConfig:
    defaults = dict(max_retries=3, jitter=False, base_delay_s=0.01)
    defaults.update(overrides)
    return RetryConfig(**defaults)


# ---------------------------------------------------------------------------
# RetryTransport (sync)
# ---------------------------------------------------------------------------


class TestRetryTransport:
    def test_returns_response_on_success(self):
        mock = MagicMock()
        mock.handle_request.return_value = httpx.Response(200)
        transport = RetryTransport(_make_config(), wrapped=mock)

        resp = transport.handle_request(httpx.Request("GET", "https://x.com/test"))

        assert resp.status_code == 200
        assert mock.handle_request.call_count == 1

    @pytest.mark.parametrize("status", RETRYABLE_STATUSES)
    @patch("oro_sdk.retry.time.sleep")
    def test_retries_on_retryable_status(self, mock_sleep: MagicMock, status: int):
        mock = MagicMock()
        mock.handle_request.side_effect = [httpx.Response(status), httpx.Response(200)]
        transport = RetryTransport(_make_config(), wrapped=mock)

        resp = transport.handle_request(httpx.Request("GET", "https://x.com/test"))

        assert resp.status_code == 200
        assert mock.handle_request.call_count == 2
        mock_sleep.assert_called_once()

    @pytest.mark.parametrize("status", NON_RETRYABLE_STATUSES)
    def test_does_not_retry_on_non_retryable_status(self, status: int):
        mock = MagicMock()
        mock.handle_request.return_value = httpx.Response(status)
        transport = RetryTransport(_make_config(), wrapped=mock)

        resp = transport.handle_request(httpx.Request("GET", "https://x.com/test"))

        assert resp.status_code == status
        assert mock.handle_request.call_count == 1

    @pytest.mark.parametrize("exc_cls", [httpx.ConnectError, httpx.ConnectTimeout])
    @patch("oro_sdk.retry.time.sleep")
    def test_retries_on_network_error(self, mock_sleep: MagicMock, exc_cls: type):
        mock = MagicMock()
        mock.handle_request.side_effect = [exc_cls("fail"), httpx.Response(200)]
        transport = RetryTransport(_make_config(), wrapped=mock)

        resp = transport.handle_request(httpx.Request("GET", "https://x.com/test"))

        assert resp.status_code == 200
        assert mock.handle_request.call_count == 2

    @patch("oro_sdk.retry.time.sleep")
    def test_exhausts_retries_returns_last_response(self, mock_sleep: MagicMock):
        mock = MagicMock()
        mock.handle_request.return_value = httpx.Response(503)
        transport = RetryTransport(_make_config(max_retries=2), wrapped=mock)

        resp = transport.handle_request(httpx.Request("GET", "https://x.com/test"))

        assert resp.status_code == 503
        assert mock.handle_request.call_count == 3  # 1 initial + 2 retries

    @patch("oro_sdk.retry.time.sleep")
    def test_exhausts_retries_raises_on_network_error(self, mock_sleep: MagicMock):
        mock = MagicMock()
        mock.handle_request.side_effect = httpx.ConnectError("fail")
        transport = RetryTransport(_make_config(max_retries=2), wrapped=mock)

        with pytest.raises(httpx.ConnectError):
            transport.handle_request(httpx.Request("GET", "https://x.com/test"))
        assert mock.handle_request.call_count == 3

    @patch("oro_sdk.retry.time.sleep")
    def test_calls_on_retry_callback(self, mock_sleep: MagicMock):
        mock = MagicMock()
        mock.handle_request.side_effect = [httpx.Response(503), httpx.Response(200)]
        on_retry = MagicMock()
        transport = RetryTransport(
            _make_config(base_delay_s=0.1, on_retry=on_retry), wrapped=mock
        )

        transport.handle_request(httpx.Request("GET", "https://x.com/test"))

        on_retry.assert_called_once()
        ctx: RetryContext = on_retry.call_args[0][0]
        assert ctx.attempt == 1
        assert ctx.status_code == 503
        assert ctx.delay_s == 0.1

    @patch("oro_sdk.retry.time.sleep")
    def test_respects_retry_after_header(self, mock_sleep: MagicMock):
        mock = MagicMock()
        mock.handle_request.side_effect = [
            httpx.Response(429, headers={"Retry-After": "2"}),
            httpx.Response(200),
        ]
        on_retry = MagicMock()
        transport = RetryTransport(
            _make_config(base_delay_s=0.1, on_retry=on_retry), wrapped=mock
        )

        transport.handle_request(httpx.Request("GET", "https://x.com/test"))

        ctx: RetryContext = on_retry.call_args[0][0]
        assert ctx.retry_after_s == 2.0
        assert ctx.delay_s == 2.0

    def test_no_retry_when_max_retries_zero(self):
        mock = MagicMock()
        mock.handle_request.return_value = httpx.Response(503)
        transport = RetryTransport(RetryConfig(max_retries=0), wrapped=mock)

        resp = transport.handle_request(httpx.Request("GET", "https://x.com/test"))

        assert resp.status_code == 503
        assert mock.handle_request.call_count == 1

    @patch("oro_sdk.retry.time.sleep")
    def test_on_retry_for_network_error_has_no_status(self, mock_sleep: MagicMock):
        mock = MagicMock()
        mock.handle_request.side_effect = [httpx.ConnectError("fail"), httpx.Response(200)]
        on_retry = MagicMock()
        transport = RetryTransport(
            _make_config(base_delay_s=0.1, on_retry=on_retry), wrapped=mock
        )

        transport.handle_request(httpx.Request("GET", "https://x.com/test"))

        ctx: RetryContext = on_retry.call_args[0][0]
        assert ctx.status_code is None


# ---------------------------------------------------------------------------
# AsyncRetryTransport — only tests that verify async plumbing;
# core retry logic is shared and already tested above.
# ---------------------------------------------------------------------------


class TestAsyncRetryTransport:
    @pytest.mark.asyncio
    async def test_returns_response_on_success(self):
        mock = MagicMock()
        mock.handle_async_request = AsyncMock(return_value=httpx.Response(200))
        transport = AsyncRetryTransport(RetryConfig(max_retries=3), wrapped=mock)

        resp = await transport.handle_async_request(httpx.Request("GET", "https://x.com/test"))

        assert resp.status_code == 200
        assert mock.handle_async_request.call_count == 1

    @pytest.mark.asyncio
    @patch("oro_sdk.retry.asyncio.sleep", new_callable=AsyncMock)
    async def test_retries_and_uses_asyncio_sleep(self, mock_sleep: AsyncMock):
        mock = MagicMock()
        mock.handle_async_request = AsyncMock(
            side_effect=[httpx.Response(429), httpx.Response(200)]
        )
        transport = AsyncRetryTransport(_make_config(), wrapped=mock)

        resp = await transport.handle_async_request(httpx.Request("GET", "https://x.com/test"))

        assert resp.status_code == 200
        assert mock.handle_async_request.call_count == 2
        mock_sleep.assert_awaited_once()

    @pytest.mark.asyncio
    @patch("oro_sdk.retry.asyncio.sleep", new_callable=AsyncMock)
    async def test_retries_on_connect_error(self, mock_sleep: AsyncMock):
        mock = MagicMock()
        mock.handle_async_request = AsyncMock(
            side_effect=[httpx.ConnectError("fail"), httpx.Response(200)]
        )
        transport = AsyncRetryTransport(_make_config(), wrapped=mock)

        resp = await transport.handle_async_request(httpx.Request("GET", "https://x.com/test"))

        assert resp.status_code == 200

    @pytest.mark.asyncio
    @patch("oro_sdk.retry.asyncio.sleep", new_callable=AsyncMock)
    async def test_exhausts_retries(self, mock_sleep: AsyncMock):
        mock = MagicMock()
        mock.handle_async_request = AsyncMock(return_value=httpx.Response(503))
        transport = AsyncRetryTransport(_make_config(max_retries=2), wrapped=mock)

        resp = await transport.handle_async_request(httpx.Request("GET", "https://x.com/test"))

        assert resp.status_code == 503
        assert mock.handle_async_request.call_count == 3

    @pytest.mark.asyncio
    async def test_no_retry_when_max_retries_zero(self):
        mock = MagicMock()
        mock.handle_async_request = AsyncMock(return_value=httpx.Response(503))
        transport = AsyncRetryTransport(RetryConfig(max_retries=0), wrapped=mock)

        resp = await transport.handle_async_request(httpx.Request("GET", "https://x.com/test"))

        assert resp.status_code == 503
        assert mock.handle_async_request.call_count == 1



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.work_item_status import WorkItemStatus


T = TypeVar("T", bound="CompleteRunResponse")


@_attrs_define
class CompleteRunResponse:
    """Response for run completion.

    Attributes:
        eval_run_id (UUID): Evaluation run ID
        status (str): New status of the run
        work_item (WorkItemStatus): Status of the work item after completion.
        agent_version_became_eligible (bool): Whether agent version just became eligible
        final_score (float | None | Unset): Final score if became eligible
    """

    eval_run_id: UUID
    status: str
    work_item: WorkItemStatus
    agent_version_became_eligible: bool
    final_score: float | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        eval_run_id = str(self.eval_run_id)

        status = self.status

        work_item = self.work_item.to_dict()

        agent_version_became_eligible = self.agent_version_became_eligible

        final_score: float | None | Unset
        if isinstance(self.final_score, Unset):
            final_score = UNSET
        else:
            final_score = self.final_score

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "eval_run_id": eval_run_id,
                "status": status,
                "work_item": work_item,
                "agent_version_became_eligible": agent_version_became_eligible,
            }
        )
        if final_score is not UNSET:
            field_dict["final_score"] = final_score

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.work_item_status import WorkItemStatus

        d = dict(src_dict)
        eval_run_id = UUID(d.pop("eval_run_id"))

        status = d.pop("status")

        work_item = WorkItemStatus.from_dict(d.pop("work_item"))

        agent_version_became_eligible = d.pop("agent_version_became_eligible")

        def _parse_final_score(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        final_score = _parse_final_score(d.pop("final_score", UNSET))

        complete_run_response = cls(
            eval_run_id=eval_run_id,
            status=status,
            work_item=work_item,
            agent_version_became_eligible=agent_version_became_eligible,
            final_score=final_score,
        )

        complete_run_response.additional_properties = d
        return complete_run_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.problem_status import ProblemStatus
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.validator_problem_result_score_components_type_0 import ValidatorProblemResultScoreComponentsType0


T = TypeVar("T", bound="ValidatorProblemResult")


@_attrs_define
class ValidatorProblemResult:
    """Result from a single validator for a specific problem.

    Attributes:
        validator_hotkey (str): Validator hotkey
        status (ProblemStatus): Status of problem evaluation.
        score (float | None | Unset): Problem score from this validator
        score_components (None | Unset | ValidatorProblemResultScoreComponentsType0): Score breakdown from this
            validator
        updated_at (datetime.datetime | None | Unset): Last update from this validator
    """

    validator_hotkey: str
    status: ProblemStatus
    score: float | None | Unset = UNSET
    score_components: None | Unset | ValidatorProblemResultScoreComponentsType0 = UNSET
    updated_at: datetime.datetime | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.validator_problem_result_score_components_type_0 import ValidatorProblemResultScoreComponentsType0

        validator_hotkey = self.validator_hotkey

        status = self.status.value

        score: float | None | Unset
        if isinstance(self.score, Unset):
            score = UNSET
        else:
            score = self.score

        score_components: dict[str, Any] | None | Unset
        if isinstance(self.score_components, Unset):
            score_components = UNSET
        elif isinstance(self.score_components, ValidatorProblemResultScoreComponentsType0):
            score_components = self.score_components.to_dict()
        else:
            score_components = self.score_components

        updated_at: None | str | Unset
        if isinstance(self.updated_at, Unset):
            updated_at = UNSET
        elif isinstance(self.updated_at, datetime.datetime):
            updated_at = self.updated_at.isoformat()
        else:
            updated_at = self.updated_at

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "validator_hotkey": validator_hotkey,
                "status": status,
            }
        )
        if score is not UNSET:
            field_dict["score"] = score
        if score_components is not UNSET:
            field_dict["score_components"] = score_components
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.validator_problem_result_score_components_type_0 import ValidatorProblemResultScoreComponentsType0

        d = dict(src_dict)
        validator_hotkey = d.pop("validator_hotkey")

        status = ProblemStatus(d.pop("status"))

        def _parse_score(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        score = _parse_score(d.pop("score", UNSET))

        def _parse_score_components(data: object) -> None | Unset | ValidatorProblemResultScoreComponentsType0:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                score_components_type_0 = ValidatorProblemResultScoreComponentsType0.from_dict(data)

                return score_components_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | ValidatorProblemResultScoreComponentsType0, data)

        score_components = _parse_score_components(d.pop("score_components", UNSET))

        def _parse_updated_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                updated_at_type_0 = isoparse(data)

                return updated_at_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        updated_at = _parse_updated_at(d.pop("updated_at", UNSET))

        validator_problem_result = cls(
            validator_hotkey=validator_hotkey,
            status=status,
            score=score,
            score_components=score_components,
            updated_at=updated_at,
        )

        validator_problem_result.additional_properties = d
        return validator_problem_result

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="WorkItemStatus")


@_attrs_define
class WorkItemStatus:
    """Status of the work item after completion.

    Attributes:
        included_success_count (int): Number of successful runs
        required_successes (int): Required successful runs
        is_closed (bool): Whether work item is closed
    """

    included_success_count: int
    required_successes: int
    is_closed: bool
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        included_success_count = self.included_success_count

        required_successes = self.required_successes

        is_closed = self.is_closed

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "included_success_count": included_success_count,
                "required_successes": required_successes,
                "is_closed": is_closed,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        included_success_count = d.pop("included_success_count")

        required_successes = d.pop("required_successes")

        is_closed = d.pop("is_closed")

        work_item_status = cls(
            included_success_count=included_success_count,
            required_successes=required_successes,
            is_closed=is_closed,
        )

        work_item_status.additional_properties = d
        return work_item_status

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any
from urllib.parse import quote
from uuid import UUID

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.agent_version_not_found_error import AgentVersionNotFoundError
from ...models.discard_request import DiscardRequest
from ...models.discard_response import DiscardResponse
from ...models.http_validation_error import HTTPValidationError
from ...models.no_active_suite_error import NoActiveSuiteError
from ...types import Response


def _get_kwargs(
    agent_version_id: UUID,
    *,
    body: DiscardRequest,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/v1/admin/agent-versions/{agent_version_id}/discard".format(
            agent_version_id=quote(str(agent_version_id), safe=""),
        ),
    }

    _kwargs["json"] = body.to_dict()

    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> AgentVersionNotFoundError | DiscardResponse | HTTPValidationError | NoActiveSuiteError | None:
    if response.status_code == 200:
        response_200 = DiscardResponse.from_dict(response.json())

        return response_200

    if response.status_code == 404:
        response_404 = AgentVersionNotFoundError.from_dict(response.json())

        return response_404

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422

    if response.status_code == 503:
        response_503 = NoActiveSuiteError.from_dict(response.json())

        return response_503

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[AgentVersionNotFoundError | DiscardResponse | HTTPValidationError | NoActiveSuiteError]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    agent_version_id: UUID,
    *,
    client: AuthenticatedClient | Client,
    body: DiscardRequest,
) -> Response[AgentVersionNotFoundError | DiscardResponse | HTTPValidationError | NoActiveSuiteError]:
    """Discard an agent version from leaderboard

     Discard an agent version from the leaderboard (reversible tombstone).

    Also cancels any active evaluation runs and closes the work item
    to prevent the validator from claiming new work for this version.

    Args:
        agent_version_id (UUID): Agent version ID
        body (DiscardRequest): Request to discard an agent version.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[AgentVersionNotFoundError | DiscardResponse | HTTPValidationError | NoActiveSuiteError]
    """

    kwargs = _get_kwargs(
        agent_version_id=agent_version_id,
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    agent_version_id: UUID,
    *,
    client: AuthenticatedClient | Client,
    body: DiscardRequest,
) -> AgentVersionNotFoundError | DiscardResponse | HTTPValidationError | NoActiveSuiteError | None:
    """Discard an agent version from leaderboard

     Discard an agent version from the leaderboard (reversible tombstone).

    Also cancels any active evaluation runs and closes the work item
    to prevent the validator from claiming new work for this version.

    Args:
        agent_version_id (UUID): Agent version ID
        body (DiscardRequest): Request to discard an agent version.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        AgentVersionNotFoundError | DiscardResponse | HTTPValidationError | NoActiveSuiteError
    """

    return sync_detailed(
        agent_version_id=agent_version_id,
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    agent_version_id: UUID,
    *,
    client: AuthenticatedClient | Client,
    body: DiscardRequest,
) -> Response[AgentVersionNotFoundError | DiscardResponse | HTTPValidationError | NoActiveSuiteError]:
    """Discard an agent version from leaderboard

     Discard an agent version from the leaderboard (reversible tombstone).

    Also cancels any active evaluation runs and closes the work item
    to prevent the validator from claiming new work for this version.

    Args:
        agent_version_id (UUID): Agent version ID
        body (DiscardRequest): Request to discard an agent version.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[AgentVersionNotFoundError | DiscardResponse | HTTPValidationError | NoActiveSuiteError]
    """

    kwargs = _get_kwargs(
        agent_version_id=agent_version_id,
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    agent_version_id: UUID,
    *,
    client: AuthenticatedClient | Client,
    body: DiscardRequest,
) -> AgentVersionNotFoundError | DiscardResponse | HTTPValidationError | NoActiveSuiteError | None:
    """Discard an agent version from leaderboard

     Discard an agent version from the leaderboard (reversible tombstone).

    Also cancels any active evaluation runs and closes the work item
    to prevent the validator from claiming new work for this version.

    Args:
        agent_version_id (UUID): Agent version ID
        body (DiscardRequest): Request to discard an agent version.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        AgentVersionNotFoundError | DiscardResponse | HTTPValidationError | NoActiveSuiteError
    """

    return (
        await asyncio_detailed(
            agent_version_id=agent_version_id,
            client=client,
            body=body,
        )
    ).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.problem_public import ProblemPublic
    from ..models.suite_public import SuitePublic


T = TypeVar("T", bound="SuiteWithProblemsResponse")


@_attrs_define
class SuiteWithProblemsResponse:
    """Response for suite with its problems.

    Attributes:
        suite (SuitePublic): Public representation of a problem suite.
        problems (list[ProblemPublic]): Problems in this suite
    """

    suite: SuitePublic
    problems: list[ProblemPublic]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        suite = self.suite.to_dict()

        problems = []
        for problems_item_data in self.problems:
            problems_item = problems_item_data.to_dict()
            problems.append(problems_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "suite": suite,
                "problems": problems,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.problem_public import ProblemPublic
        from ..models.suite_public import SuitePublic

        d = dict(src_dict)
        suite = SuitePublic.from_dict(d.pop("suite"))

        problems = []
        _problems = d.pop("problems")
        for problems_item_data in _problems:
            problems_item = ProblemPublic.from_dict(problems_item_data)

            problems.append(problems_item)

        suite_with_problems_response = cls(
            suite=suite,
            problems=problems,
        )

        suite_with_problems_response.additional_properties = d
        return suite_with_problems_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
